if(!dojo._hasResource["com.ibm.data.JsonStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.JsonStore"] = true;
// A json store that can be used for the customize shelf
// dojo.data.ItemFileReadstore already is a JSON store, this store extends it and overwrites the _fetchItems function to do an AND instead of an OR search for the given keywords
// it also adds support for passing in parameters for an nls bundle and keywords

dojo.provide("com.ibm.data.JsonStore");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.data.util.filter");
dojo.require("com.ibm.data.ProxyHelper");
dojo.require("com.ibm.data.CatalogMixin");
dojo.require("dojo.i18n");

dojo.declare("com.ibm.data.JsonStore", 
		[dojo.data.ItemFileReadStore, com.ibm.data.ProxyHelper, com.ibm.data.CatalogMixin],
		{	
			nlsBundle: null,
		
			_getItemsFromLoadedData: function(/* Object */ dataObject){
				this.inherited("_getItemsFromLoadedData",arguments);
				
				this.shelfBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
				var localizationPackageName = dataObject.localizationPackageName;
				var localizationBundleName = dataObject.localizationBundleName;
				if(localizationPackageName && localizationBundleName){
					this.nlsBundle = dojo.i18n.getLocalization(localizationPackageName, localizationBundleName);
					dojo.forEach(this._arrayOfAllItems,
						dojo.hitch(this, function(item) {
							if (item.label) {
								item.label[0] = (this.nlsBundle && this.nlsBundle[item.label[0]]) ? this.nlsBundle[item.label[0]] : item.label[0];
							}
							else {
								item.label = [];
							}
							if (!item.label[0]) {
								item.label[0] = this.shelfBundle.add_content_untitled;
							}
							if (item.description) {
								item.description[0] = (this.nlsBundle && this.nlsBundle[item.description[0]]) ? this.nlsBundle[item.description[0]] : item.description[0];
							}
							if (item.help) {
								item.help[0] = (this.nlsBundle && this.nlsBundle[item.help[0]]) ? this.nlsBundle[item.help[0]] : item.help[0];
							}
						})
					);
				}
			},
			
			mapItem: function(/*DatstoreItem*/item){
				var map = {};
			
				map.label = this.getValue(item, "label", null);
				map.description = this.getValue(item, "description", null);
				map.url = this.getValue(item, "url", null);
				map.id = this.getValue(item, "id", null);
				map.thumbnail = this.getValue(item, "thumbnail", null);
				map.help = this.getValue(item, "help", null);
				map.takeoverFcn = this.getValue(item,"takeover",null);
				
				return map;
			},
		
			_fetchItems: function(/*Object*/ keywordArgs, /*Function*/ findCallback, /*Function*/ errorCallback){
				var self = this;
				var filter = function(requestArgs, arrayOfItems){
					
					var items = [];
					var i = 0;
					if(requestArgs.query){
						var filterKeywords = requestArgs.query.keywords;
						var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 
					
						var regexpList = {};
						var currFilter = null;
						for(i = 0; i < filterKeywords.length; i++){
							currFilter = filterKeywords[i];
							if(typeof currFilter === "string"){
								regexpList[currFilter] = dojo.data.util.filter.patternToRegExp("*"+currFilter+"*", ignoreCase);
							}
						}

						for(i = 0; i < arrayOfItems.length; ++i){
							var match = true;
							var candidateItem = arrayOfItems[i];
							if(candidateItem === null){
								match = false; 
							} else {
								for(var j = 0; j < filterKeywords.length; j++){
									currFilter = filterKeywords[j];
									if(!self._containsValue(candidateItem, 'label', currFilter, regexpList[currFilter]) && 
											!self._containsValue(candidateItem, 'description', currFilter, regexpList[currFilter])){
										match = false;
									}
								}
							}
							
							if(match){
								items.push(candidateItem);
							}
						}
						findCallback(items, requestArgs);
					} else {
						for(i = 0; i < arrayOfItems.length; ++i){
							var item = arrayOfItems[i];
							if(item !== null){
								items.push(item);
							}
						}
						findCallback(items, requestArgs);
					}
			};

			if(this._loadFinished){
				filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
			} else {

				if(this._jsonFileUrl){
					if(this._loadInProgress){
						this._queuedFetches.push({args: keywordArgs, filter: filter});
					} else {
						this._loadInProgress = true;
						var getArgs = {
								url: self._jsonFileUrl, 
								handleAs: "json-comment-optional"
						};
						var getHandler = dojo.xhrGet(getArgs);
						getHandler.addCallback(function(data){
							try{
								self._getItemsFromLoadedData(data);
								self._loadFinished = true;
								self._loadInProgress = false;
				
								filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
								self._handleQueuedFetches();
							}catch(e){
								self._loadFinished = true;
								self._loadInProgress = false;
								errorCallback(e, keywordArgs);
							}
						});
						getHandler.addErrback(function(error){
							self._loadInProgress = false;
							errorCallback(error, keywordArgs);
						});
					}
				} else if(this._jsonData){
					try{
						this._loadFinished = true;
						this._getItemsFromLoadedData(this._jsonData);
						this._jsonData = null;
						filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
					}catch(e){
						errorCallback(e, keywordArgs);
					}
				} else {
					errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
				}
			}
		}
});

}
