if(!dojo._hasResource["com.ibm.mm.builder.iwidget.navigation.NavigationStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.iwidget.navigation.NavigationStore"] = true;
dojo.provide("com.ibm.mm.builder.iwidget.navigation.NavigationStore");

dojo.require("dojo.data.ItemFileReadStore");

dojo.declare("com.ibm.mm.builder.model.NavigationStore", dojo.data.ItemFileReadStore, {
    /**
     * summary:
     *  NavigationNodesReadStore extends ItemFileReadStore to implement an 
     *  navigation model store in Lotus Mashups navigation nodes.
     */
    constructor: function(navigationModel, spaceId, pageId){
		this.navigationModel = navigationModel;
		this.navigationUtil = com.ibm.mm.builder.utils.navigationUtil;
		this.pageUtil = com.ibm.mm.builder.utils.pageUtil;
		this.acModeUtil = com.ibm.mm.builder.utils.acModelUtil;
		
        this.items = [];
		
		this.canSeeHiddenPages = this.acModeUtil.isPageOwner(pageId) || 
								this.acModeUtil.getUserPermission(pageId) === this.acModeUtil.EDIT;
		
		var root = this.navigationUtil.getNavigationRootNode(spaceId);
		this.items.push(root);
		
		var childrenIter = this.navigationModel.getChildren(root);
		while(childrenIter.hasNext()) {
			var childNode = childrenIter.next();
			if (this.pageUtil.isVisiable(childNode) || this.canSeeHiddenPages) {
				this.items.push(childNode);
			}
		}
		
        //convert to a map
        this.parent_node_map = {};
        this.id_node_map = {};
        this.rootNode = root;
		root.__type = "root";
		
        dojo.forEach(this.items, function(node){
			var parent = this.navigationModel.getParent(node.getID()).start();
			var parentId = parent ? parent.getID() : null;
			if (parentId) {
				if (!this.parent_node_map[parentId]) {
					this.parent_node_map[parentId] = [];
				}
				this.parent_node_map[parentId].push(node);
				this.id_node_map[node.getID()] = node;
			}
        }, this);
        dojo.forEach(this.items, function(node){
            node.__children = this.parent_node_map[node.getID()];
			
			var hasChildren = this.navigationModel.hasChildren(node);
			node.__type = hasChildren ? "_stub" : "node";
        }, this);
    },
    
    //dojo.data.api.Read API
    getValue: function( /* item */ item, /* attribute-name-string */ attribute, 
                        /* value? */ defaultValue){
        var values = this.getValues(item, attribute);
        return (values.length > 0)?values[0]:defaultValue;
    },

    getValues: function(/* item */ item, /* attribute-name-string */ attribute){
		var values = null;
		switch(attribute) {
			case "id":
			    values = [item.getID()];
			    break;
			case "title":
			    if (this.rootNode === item) {
					values = ["  "];
				} else {
					values = [this.pageUtil.getUnescapedPageTitle(item.getID())];
				}
				break;
			case "description":
			    values = [this.pageUtil.getPageDescription(item.getID())];
				break;
			case "type":
			    values = item.__type;
				break;
			case "parentId":
				var parent = this.navigationModel.getParent(item.getID()).start();
	            var parentId = parent ? parent.getID() : null;
			    values = [parentId];
				break;
			case "children":
			    values = item.__children;
				break;
			case "selfUri":
			    values = [item.getID()];
				break;
			default:
			    values = null;
		}
		values = (values && dojo.isArray(values)) ? values : [values];
		return values;
    },

    getAttributes: function(/* item */ item){
        return ["id", "title", "description", "type", "parentId", "children", "selfUri"];
    },

    hasAttribute: function( /* item */ item, /* attribute-name-string */ attribute){
        return !!(this.getValue(item, attribute));
    },

    containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
        return this.getValue(item, attribute) == value;
    },

    isItem: function(/* anything */ something){
		return something.getID();
    },

    isItemLoaded: function(/* anything */ something) {
		return this.isItem(something) && something.__type !== "_stub";
    },

    /**
     * Lazy load sub items
     * @param {Object} keywordArgs
     */
    loadItem: function(/* object */ keywordArgs){
		var item = keywordArgs.item;
        this._assertIsItem(item);
		
		var childrenIter = this.navigationModel.getChildren(item);
		item.__children = [];
		while(childrenIter.hasNext()) {
            var childNode = childrenIter.next();
            if (this.pageUtil.isVisiable(childNode) || this.canSeeHiddenPages) {
				if (!childNode.__type) {
					childNode.__type = this.navigationModel.hasChildren(childNode) ? "_stub" : "node";
				}
				var parent = this.navigationModel.getParent(childNode.getID()).start();
				var parentId = parent ? parent.getID() : null;
				if (!this.parent_node_map[parentId]) {
					this.parent_node_map[parentId] = [];
				}
				this.parent_node_map[parentId].push(childNode);
				this.id_node_map[childNode.getID()] = childNode;
				this.items.push(childNode);
				item.__children.push(childNode);
			}
        }
		item.__type = "node";
		
	    //Done processing!  Call the onItem, if any.
        if(keywordArgs.onItem){
            var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
            keywordArgs.onItem.call(scope, item);
        }
    },

    getFeatures: function(){
        return {'dojo.data.api.Read':true, 'dojo.data.api.Identity':'id'};
    },

    close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
    },

    getLabel: function(/* item */ item){
        return this.getValue(item, "title");
    },

    getLabelAttributes: function(/* item */ item){
        return ["title"];
    },
    
    //dojo.data.api.Identity API
    getIdentity: function(/* item */ item){
        return this.getValue(item, "id");
    },

    getIdentityAttributes: function(/* item */ item){
        return ["id"];
    },
	
    fetchItemByIdentity: function(/* object */ keywordArgs){
        var node = this.id_node_map[keywordArgs.identify];
        node = node ? node : null;
        if(keywordArgs.onItem){
            var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
            keywordArgs.onItem.call(scope, node);
        }
    },
    
    fetch: function(/* Object */ keywordArgs){
        var query = keywordArgs.query;
        if(query === null){
            keywordArgs.onComplete([this.rootNode], keywordArgs);
        }
    },
    
    getRootItem: function(){
        return this.rootNode;
    },
	
	reset: function() {
		dojo.forEach(this.items, function(item) {
			delete item.__type;
            delete item.__children;
		});
	}
});

}
