// This is a container which follows the one ui lotus palette spec
// must be provided with a processChange function
dojo.provide("com.ibm.pb.customize.LotusPaletteContainer");

dojo.require("com.ibm.data.CatalogFeedStore");
dojo.require("com.ibm.data.MashupFileDataStore");
dojo.require("com.ibm.data.JsonStore");
dojo.require("com.ibm.customize.ResultsList");
dojo.require("com.ibm.customize.SearchBar");
dojo.require("com.ibm.customize.Menu");
dojo.require("dijit.Dialog");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "Shelf");
dojo.requireLocalization("com.ibm.bundles", "Theme");

dojo.declare("com.ibm.pb.customize.LotusPaletteContainer",
	[dijit._Widget, dijit._Templated],
	{
		jsonFile: null,				// a JSON configuration file
		dialog: null,				// a dijit dialog
		maxLabelLength: 16,			// the maximum length allowed for a label
		smallWidget: false,			// false if it follows the lotusLargeWidget pattern
		numRows: 4,					// the number of rows of results to be displayed
		itemLimit: 9,
		label: "",					// a label for this palette section
		selectedClass: "lotusSelected",
		imagePath: ".",
		draggable: false,			// true if items can be dragged off the shelf, will only work with Portal dnd
		size: null,
		dndType: "ibmPortalShelfItem",		// dnd type of items that are draggable
		templateString:'<div class="lotusui">' +
							'<div class="lotusPaletteBody">' +
								'<div dojoAttachPoint="left" class="lotusPaletteNav"></div>' +
								'<div dojoAttachPoint="center" class="lotusPaletteContent lotusui"></div>' +
							'</div>' +
						'</div>',
						
		constructor: function() {
			this.nlsBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		},
		
		postMixInProperties: function() {
			if(this.smallWidget) {
				this.dialog = new com.ibm.pb.customize.LotusDialog({ blankImg:(this.blankImg)?this.blankImg:this.imagePath + "/blank.gif" });
				this.dialog.closeButtonNode.onclick = this.dialog.closeLinkNode.onclick = dojo.hitch(this,function(){this.dialog.cancel();return false;});
			}
		},
		
		processChange: function(/*Object*/args) {
		},
		
		getSize: function(){
			// summary: determine the width of a palette widget element on the page with all the current settings by adding one to the page, taking its width, then deleting it
			// use the palette widget to determine how many of them can fit in the ResultsList
			if(this.size) { return this.size; }
			var paletteWidget = dojo.create("div",{className:"lotusPaletteWidget"});
			document.body.appendChild(paletteWidget);
			if (paletteWidget.currentStyle){
				marginL = paletteWidget.currentStyle["marginLeft"].match(/^\d+/);
				marginR = paletteWidget.currentStyle["marginRight"].match(/^\d+/);
			} else if (window.getComputedStyle) {
				marginL = document.defaultView.getComputedStyle(paletteWidget,null).getPropertyValue("margin-left").match(/^\d+/);
				marginR = document.defaultView.getComputedStyle(paletteWidget,null).getPropertyValue("margin-right").match(/^\d+/);
			} 
			var width = paletteWidget.offsetWidth + marginL*1 + marginR*1;
			this.left.innerHTML = ".";
			var containerWidth = this.center.offsetWidth;
			this.left.innerHTML = "";
			var resultsInOneRow = Math.floor(containerWidth / width);
			dojo.destroy(paletteWidget);
			this.size = resultsInOneRow * this.numRows;
			return this.size || this.itemLimit;
		},
		
		destroyChildWidgets: function(){
			if(this.resultsList) {
				this.resultsList.destroy();
				this.resultsList = null;
			}
			if(this.searchBar) {
				this.searchBar.destroy();
				this.searchBar = null;
			}
			if(this.menu) {
			this.menu.destroy();
			this.menu = null;
			}
			this.started = false;
			this.inherited(arguments);
		},
		
		destroy: function(){
			this.destroyChildWidgets();
			this.inherited(arguments);
		},
		
		setupUI: function(){
			// summary: starts the UI setup
			this.processJSON(dojo.hitch(this,this.setupUICallback));
		},
		
		setupUICallback: function(sources){
			// summary: populates the SearchBar, Menu and ResultsList widget with configuration data from the JSON file
			this.resultsList = new com.ibm.customize.ResultsList({parent:this,size:this.getSize(),draggable:this.draggable,dndType:this.dndType});
			this.center.appendChild(this.resultsList.domNode);
				
			this.searchBar = new com.ibm.customize.SearchBar({searchId:this.id,imagePath:this.imagePath});
			this.left.appendChild(this.searchBar.domNode);
				
			this.menu = new com.ibm.customize.Menu({sources:sources,searchId:this.id,menuTitle:this.label,blankImg:(this.blankImg)?this.blankImg:this.imagePath + "/blank.gif"});
			this.left.appendChild(this.menu.domNode);
		},
		
		processJSON: function(/*function*/callback){
			// summary: processes the JSON file associated with this controller
			// callback: function to call after the JSON is processed that uses the sourceObjects
			dojo.xhrGet ({
		        url: this.jsonFile,
		        handleAs: 'json',
		        load: dojo.hitch(this, function(data){
					// get the bundle if one is provided
					if(data.localizationPackageName && data.localizationBundleName){
						this.nlsJsonBundle = dojo.i18n.getLocalization(data.localizationPackageName, data.localizationBundleName);
					} else this.nlsJsonBundle = null;
					var sources = data.categories;
					var sourceObjects = [];
					for(var i = 0, l = sources.length; i < l; i++) {
						if(sources[i].categories){
							var childSourceObjects = [];
							var childSources = sources[i].categories;
							for(var j = 0, l2 = childSources.length; j < l2; j++){
								var source = this.processCategory(childSources[j]);
								if(source) childSourceObjects.push(source);
							}
							if(childSourceObjects.length > 0) {
								var label = this.getNlsString(sources[i].label,this.nlsBundle["add_content_untitled"]);
								sourceObjects.push({label:label,categories:childSourceObjects});
							}
						} else {
							var source = this.processCategory(sources[i]);							
							if(source) sourceObjects.push(source);
						}
					}
					if(callback) callback(sourceObjects);
				}),	
		        error: dojo.hitch(this, function (/*String*/error) {
					console.error("error loading categories: " +error);
		            var errorMsg = this.createLotusMsgNode(this.nlsBundle["shelf_configError"],"Error");
					dojo.style(errorMsg,"margin","20px");
					this.domNode.innerHTML = "";
					this.domNode.appendChild(errorMsg);
		        })
		    });
		},
		
		processCategory: function(source){
			// pass the exact same sources item to the class's constructor minus the datastore and renderers properties
			var map = dojo.mixin({}, source);
			if(map.url && this.createProxyURI) map.url = this.createProxyURI(map.url);
			delete map.datastore; delete map.renderers; delete map.label;
			// create a datastore hub and a new source object
			var hub = new (dojo.getObject(source.datastore))(map);
			
			if(!hub.validateSelf()) return null;  // do not use this source if it does not validate
			if (source && source.visibilityFn){ // Run optional visibility function for category
				var visibility = dojo.getObject(source.visibilityFn);
				if (visibility && dojo.isFunction(visibility) && !visibility()){
					return null;
				}
			}
			if(hub.getSourceLabel()) var label = hub.getSourceLabel();  // get dynamic label
			else var label = this.getNlsString(source.label,this.nlsBundle["add_content_untitled"]);
			var searchStr = this.getNlsString(source.searchStr,null);
			return {label:label,hub:hub,renderers:source.renderers,url:source.url,searchStr:searchStr};
		},
		
		getNlsString: function(key, defaultStr){
			var value = this.nlsJsonBundle && this.nlsJsonBundle[key] ? this.nlsJsonBundle[key] : key;
			if(value && value.length == 0) value = defaultStr;
			return value;
		},
		
		newCell: function(/*Object*/source,/*DataItem*/item){
			// summary:  override this function to fill innerCell with information about the given datastore item
			// source: a datastore
			// item: an item in source
			var map = source.hub.mapItem(item);
			
			var label = (map.label.length > this.maxLabelLength)?map.label.substring(0,this.maxLabelLength) + '...':map.label;
			if(this.smallWidget) {
				var applyToPage = dojo.hitch(this, function(cID,sID) { this.processIntermediateDialogs(map,source,item,cID,sID); });
				showDetails = dojo.hitch(this, function() { this.showDetailsDialog(map,applyToPage); return false; /* to prevent onBeforeUnload event from triggering in IE */ });
				
				var context = {hub:source.hub, item:item, map:map, truncTitleText:label, imagePath:this.imagePath, apply:applyToPage, showDetails:showDetails};
				if(source.hub.noIcons){
					var widget = new com.ibm.pb.customize.noIconSmallWidget(context);
				} else if(source.hub.namespace == "feed") {
					var widget = new com.ibm.pb.customize.feedSmallWidget(context);
				} else {
					if(ibmCfg.portalConfig.portalContext) {
						var widget = new com.ibm.pb.customize.smallWidget(context);
					} else {
						var widget = new com.ibm.pb.customize.smallMenuWidget(context);
					}
				}
			}
			else {
				var applyToPage = dojo.hitch(this, function() { this.processChange(dojo.mixin(map,{hub:source.hub, item:item})) });
				var widget = new com.ibm.pb.customize.largeWidget({map:map, truncTitleText:label, imagePath:this.imagePath, apply:applyToPage, current:map.id == this.current});
			}
			return widget.domNode;
		},
		
		createLotusMsgNode: function(/*string*/message,/*string*/type){
			// summary: returns a div node containing the given message with OneUI styles
			// msg: the error message to display
			// type: either "Warning", "Error", "Info" or "Confirm"
			var err = dojo.create("div",{className:"lotusMessage lotus" + type, style:{ marginRight:"15px"}, role: "alert"});
			var msg = dojo.create("span",{innerHTML:message});
			err.appendChild(msg);
			return err;
		},
		
		changeSelected: function(/*string*/selected){
			// reset the look of the previously selected style/layout
			var items = dojo.query(".lotusPaletteWidget",this.center), selectedWidget = null;
			dojo.forEach(items, dojo.hitch(this, function(item){
				selectedWidget = dijit.byId(item.id);
				if(selectedWidget) selectedWidget.deselect();
			}));
			selectedWidget = dijit.byId(selected);
			if(selectedWidget) selectedWidget.select();
		},
		
		/**********************************/
		/*        DIALOG HELPERS          */
		/**********************************/
		
		processIntermediateDialogs: function(/*MapItem*/map,/*Object*/source,/*DataItem*/item,/*string*/cID,/*string*/sID){
			// source: a datastore
			// item: an item in source's hub
			if(source.renderers.length == 0){
				// if there are no renderers defined, then this is a self renderer and we assume the item's id is the id of the piece of content to be added
				var takeoverObject = map.takeoverFcn ? {submit:dojo.hitch(this,this.processChange),getInfo:dojo.hitch(this,this.getValuesFromDialog),showInfo:dojo.hitch(this,this.showWaitMsgInDialog),submitObj:{hub:source.hub,item:item,sID:sID,cID:cID}} : {};
				var args = dojo.mixin(map,{takeoverObject:takeoverObject,sID:sID,cID:cID,hub:source.hub,item:item});
				this.processChange(args);
			} else if(source.renderers.length == 1){ 
				// if there is only one renderer available, add it
                var dataLabel = map.label;
				var args = dojo.mixin(dojo.mixin(map,source.renderers[0]),{label:dataLabel,prefsMapFcn:source.renderers[0].fcn,sID:sID,cID:cID,hub:source.hub,item:item});
				this.processChange(args);
			} else {
				// an intermediate dialog box is opened for the user to choose the desired renderer
				var content = dojo.create("div");
				var header = dojo.create("h4");
				var headerLabel = dojo.create("label",{"for":"widgetRendererSelect",innerHTML:this.nlsBundle["add_content_selectRenderer"]});
				header.appendChild(headerLabel);
				content.appendChild(header);
				var rendererSelect = dojo.create("select",{id:"widgetRendererSelect"});
				content.appendChild(rendererSelect);
				for(var i = 0; i < source.renderers.length; i++){
					var select = dojo.create("option",{innerHTML:this.nlsJsonBundle && this.nlsJsonBundle[source.renderers[i].label] ? this.nlsJsonBundle[source.renderers[i].label] : source.renderers[i].label,value:i});
					rendererSelect.appendChild(select);
				}
				var onSubmit = dojo.hitch(this, function(){ 
					this.dialog.cancel();
					var renderer = source.renderers[rendererSelect.value];
                    var dataLabel = map.label; 
					var args = dojo.mixin(dojo.mixin(map,renderer),{label:dataLabel,prefsMapFcn:renderer.fcn,sID:sID,cID:cID,hub:source.hub,item:item});
					this.processChange(args);
				});
				this.dialog.setContent(content, this.nlsBundle["add_content_addToPage"], this.nlsBundle["add_content_add"], onSubmit);
			}
		},
		
		showDetailsDialog: function(/*Object*/itemMap,/*Function*/submitFcn){
			// summary: prepares the markup for the popup dialog to display details.  
			// itemMap: a map of information on the content from the mapItem function of the CatalogMixin class
			// submitFcn: the function to call when the submit button on the dialog is clicked
			var content = dojo.create("div");
			var title = dojo.create("h4",{innerHTML:(itemMap.label)?itemMap.label:this.nlsBundle["add_content_untitled"]});
			content.appendChild(title);
			if(itemMap.updated) {
				var updated = dojo.create("p",{innerHTML:dojo.string.substitute(this.nlsBundle["add_content_lastMod"], [itemMap.updated])});
				content.appendChild(updated);
			}
			if(itemMap.description){ 
				var desc = dojo.create("p",{innerHTML:itemMap.description});
				content.appendChild(desc);
			}
			if(itemMap.tags && itemMap.tags.length > 0){ 
				var tags = dojo.create("p",{innerHTML:dojo.string.substitute(this.nlsBundle["add_content_tags"], [itemMap.tags.join(", ")])});
				content.appendChild(tags);
			}
			var submit = dojo.hitch(this, function(){ submitFcn(); });
			this.dialog.setContent(content, this.nlsBundle["add_content_details"], this.nlsBundle["add_content_addToPage"], submit);
		},
		
		showWaitMsgInDialog: function(/*String*/message){
			// summary: abstracts the use of the popup dialog by displaying a message
			// message: message to display in the dialog
			if(!this.waitingDialog)
			this.waitingDialog = new com.ibm.pb.customize.LotusWaitingDialog({spinner:builderConfig.builderPath + "/images/loading.gif", message:message});
			try {
				this.waitingDialog.show();
			} catch(e) {}
			return dojo.hitch(this,function(){ this.waitingDialog.hide(); });
		},
		
		getValuesFromDialog: function(/*object*/formObjs,/*String*/titleText,/*String*/submitText,/*String*/errorText,/*function*/callback){ 
			// summary: abstracts the use of the popup dialog by dislaying a form of text fields to be filled out
			// formObjs: an array of objects in the form:	label: a label for a text field 
			//												id: an id for the input field
			//												value: a value for the same text field 
			//												regex: a regular expression that the value of the field must validate against
			//												error: an error to display if the value does not validate
			// titleText: text to user as the title of the dialog
			// submitText: text to user on the submit button in the dialog
			// errorText: an optional error message to preface the entire form in the dialog, ie a "try again" error
			// callback: a function to be called when all the fields validate, it is called with an object of key value pairs
			//			 with the labels from the formObjs as keys and their corresponding values from the form as values
			var content = document.createElement("div");
			for(var i = 0; i < formObjs.length; i++){ // add an input for each item in formObjs
				var title = dojo.create("h4");
				var titleLabel = dojo.create("label",{innerHTML:formObjs[i].label, "for":formObjs[i].id});
				var error = dojo.create("div",{className:"lotusFormError"}); formObjs[i].errorDiv = error;
				var additionalClasses = "";
				if (formObjs[i].ltr)
					additionalClasses =" lotusLTR";
				var input = dojo.create("input",{type:"text", id:formObjs[i].id, className:"lotusText" + additionalClasses, value:formObjs[i].value}); formObjs[i].input = input;
				title.appendChild(titleLabel);
				content.appendChild(error);
				content.appendChild(title);
				content.appendChild(input);
			}
			if(errorText) formObjs[0].errorDiv.innerHTML = errorText;
			var validateInputs = dojo.hitch(this, function(){
				var fail = false;
				var values = {};
				for(var i = 0; i < formObjs.length; i++){
					if(!formObjs[i].input.value.match(formObjs[i].regex)){ // if the input does not match the regular expression, display an error
						fail = true;
						formObjs[i].errorDiv.innerHTML = formObjs[i].error;
					} else { // if the input does match the regex, add the input value to the return object
						values[formObjs[i].id] = formObjs[i].input.value;
						formObjs[i].errorDiv.innerHTML = "";
					}
				}
				if(!fail){ // if all the regular expressions matched, run the callback function on the values and close the dialog
					this.dialog.cancel();
					callback(values);				
				}
			});
			this.dialog.setContent(content, titleText, submitText, dojo.hitch(this, function(){ validateInputs(); }));
		}
});

dojo.provide("com.ibm.pb.customize.smallWidget");
dojo.declare("com.ibm.pb.customize.smallWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		map: {},
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
										'${truncTitleText}' +
										'<span class="ibmPortalHiddenText ibmHiddenText wpthemeAccess">${detailDialogInstruction}</span>' +
									'</a>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" role="button" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt="${addToPage}"/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				//this.map.thumbnail = this.imagePath + "/portletIcon.gif";
				this.map.thumbnail = builderConfig.builderPath + "/images/generic_widget_icon.gif"; 
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.blankPath = this.imagePath + "/blank.gif";
		},
			
		postCreate: function(){ 
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
			
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});
	
dojo.provide("com.ibm.pb.customize.smallMenuWidget");
dojo.declare("com.ibm.pb.customize.smallMenuWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		hub: null,
		item: null,
		map: {},
		warningIcon: "",
		menu: null,
		isFeed: false,
		isOpenSocial: false,
		widgetActions: "Actions",
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<a class="lotusWarningIcon" href="javascript:void(0)" title="sandboxed" dojoAttachPoint="warningIconAnchor">' +
										'<img width="16" height="16" src="${blankPath}" alt=""/>' +
										'<span class="lotusAltText">sandboxed</span>' +
							    '</a>' +
								'<div class="lotusTitle">' +
									'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
										'${truncTitleText}' +
										'<span class="ibmPortalHiddenText ibmHiddenText wpthemeAccess">${detailDialogInstruction}</span>' +
									'</a>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt="${addToPage}"/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
								'<a class="lotusIcon iconContextMenu" href="javascript:void(0)" title="${widgetActions}" dojoAttachPoint="contextMenuButton">' +
								    '<img width="16" height="16" src="${menuIconPath}" alt="${widgetActions}"/>' +
								    '<span class="lotusAltText">${widgetActions}</span>' +
							    '</a>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.widgetActions = bundle["add_content_widgetActions"] || this.widgetActions; 
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				//this.map.thumbnail = this.imagePath + "/portletIcon.gif";
				this.map.thumbnail = builderConfig.builderPath + "/images/generic_widget_icon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			if (!this.warningIcon) this.warningIcon = "/mum/images/oneui/iconWarning.gif";
			this.blankPath = this.imagePath + "/blank.gif";
			this.menuIconPath = "/mum/images/context_menu.png";
		},
			
		postCreate: function(){ 
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
			if(!ibmCfg.portalConfig.portalContext) {  // Portal does not use the menus
				var applyToPage = dojo.hitch(this, function(){
					this.apply();
				});
				var updateShelfItemUI = dojo.hitch(this, function(){
					this._updateShelfItemUI(true);
				});

				if (this._isFromHub()) {
					this.isOpenSocial = this.map.widgetType == "mm_OSGadget";
				} else {
					this.isOpenSocial = this.hub.category == "OpenSocial";
				};
				this.isFeed = this.item.store.namespace == "feed";
				
				this.widgetContextMenu = new com.ibm.mm.builder.iwidget.SmallWidgetContextMenu({
					targetNodeIds: [this.contextMenuButton],
					entryId: this.map.id,
					category: this.category,
					widget: this,
					addWidget: applyToPage,
					updateWidgetItemUI: updateShelfItemUI
				});
				this._updateShelfItemUI(false);
			}
		},
		
		_isFromHub: function() {
			return dojo.trim(ibmConfig["com.ibm.mashups.builder.contentShelf.mode"]) == "hub";
		},
		
		_updateShelfItemUI: function(refresh){
			var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
			var sandboxEnabled = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.SANDBOXENABLED);
			if (!sandboxEnabled) return;
			
			if (refresh) {
				this.map.isTrusted = this.widgetContextMenu.widget.map.isTrusted;
            }
			
			if (this._isFromHub()) {	
				this.map.isTrusted = this.map.sandbox_definition == "true" ? false : true;
				if (this.map.isTrusted) {
					this.map.isTrusted = this.map.sandbox_user == "true" ? false : true;
				}	
			} else {		
				this.map.isTrusted = com.ibm.mm.builder.utils.iwidgetUtil.isWidgetTrusted(this.map.id);	
			}			
            
			if(this.map.isTrusted){
				dojo.removeClass(this.lotusPaletteWidget, "lotusPaletteUnTrustedWidget");
			} else {
				dojo.addClass(this.lotusPaletteWidget, "lotusPaletteUnTrustedWidget");
			}
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			if(this.widgetContextMenu) {
				this.widgetContextMenu._clearMenu();
				this.widgetContextMenu.destroy();
			}
			this.inherited(arguments);
		}
	});

dojo.provide("com.ibm.pb.customize.feedSmallWidget");
dojo.declare("com.ibm.pb.customize.feedSmallWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		hub: null,
		item: null,
		map: {},
		warningIcon: "",
		menu: null,
		detailDialogInstruction: "press enter to see details dialog",
		templateString: '<div class="lotusPaletteWidget" dojoAttachPoint="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">' +
										'${truncTitleText}' +
										'<span class="ibmPortalHiddenText ibmHiddenText wpthemeAccess">${detailDialogInstruction}</span>' +
									'</a>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt=""/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>',
			
		postMixInProperties: function(){
			var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.addToPage = bundle["add_content_addToPage"];
			this.detailDialogInstruction = bundle["detail_dialog_instruction_text"] || this.detailDialogInstruction;
			if (!this.map.thumbnail) {
				//this.map.thumbnail = this.imagePath + "/portletIcon.gif";
				this.map.thumbnail = builderConfig.builderPath + "/images/generic_widget_icon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.blankPath = this.imagePath + "/blank.gif";
		},
		
		postCreate: function(){	
			this.titleAnchor.onclick = dojo.hitch(this, function(){ this.showDetails(); return false; });
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
		},
		
		apply: function(){},
		
		showDetails: function(){},
		
		select: function(){},
		
		deselect: function(){},
		
		destroy: function() {
			this.titleAnchor.onclick = null;
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});

dojo.provide("com.ibm.pb.customize.noIconSmallWidget");
dojo.declare("com.ibm.pb.customize.noIconSmallWidget",
	[com.ibm.pb.customize.smallWidget],
	{
		templateString: '<div class="lotusPaletteWidget">' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon"></div>' +
								'<div class="lotusTitle">' +
									'<a href="javascript:void(0);" title="${map.label}" dojoAttachPoint="titleAnchor">${truncTitleText}</a>' +
								'</div>' +
								'<a class="lotusIcon" href="javascript:void(0)" title="${addToPage}" role="button" dojoAttachPoint="addButton">' +
									'<img width="16" height="16" src="${blankPath}" alt=""/>' +
									'<span class="lotusAltText">+</span>' +
								'</a>' +
							'</div>' +
						'</div>'
	});
	
	dojo.provide("com.ibm.pb.customize.largeWidget");
	dojo.declare("com.ibm.pb.customize.largeWidget",
	[dijit._Widget, dijit._Templated],
	{
		truncTitleText: "",
		imagePath: ".",
		map:{},
		current: false,
		selectedClass: "lotusSelected",
		hiddenSelectedText: "currently selected for the page",
		hiddenUnselectedText: "press enter to select",
		templateString: '<div class="lotusPaletteWidget" dojoAttachPoint="addButton" >' +
							'<div class="lotusInner">' +
								'<div class="lotusIcon">' +
									'<span>' +
										'<img alt="${truncTitleText}" src="${map.thumbnail}"/>' +
									'</span>' +
								'</div>' +
								'<div class="lotusTitle">' +
									'<a href="javascript:void(0);" title="${map.label}">' +
										'${truncTitleText}' +
										'<span class="ibmPortalHiddenText ibmHiddenText wpthemeAccess" dojoAttachPoint="hiddenSelectedTextNode">${hiddenUnselectedText}</span>' +
									'</a>' +
								'</div>' +
							'</div>' +
						'</div>',
						
		postMixInProperties: function(){
			if (!this.map.thumbnail) {
				//this.map.thumbnail = this.imagePath + "/portletIcon.gif";
				this.map.thumbnail = builderConfig.builderPath + "/images/generic_widget_icon.gif";
			} else {
				this.map.thumbnail = com.ibm.mashups.enabler.utils.EndpointHelper.resolve(this.map.thumbnail);
			}
			this.hiddenSelectedText = dojo.i18n.getLocalization("com.ibm.bundles", "Theme")["currently_selected"] || this.hiddenSelectedText;
			this.hiddenUnselectedText = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf")["selection_instructions"] || this.hiddenUnselectedText;
			this.id = this.map.id;
		},
		
		postCreate: function(){
			if(this.current) this.select();
			this.addButton.onclick = dojo.hitch(this, function(){ this.apply(); return false; });
		},
		
		apply: function(){},
		
		select: function(){
			this.current = true;
			dojo.addClass(this.domNode,this.selectedClass);
			this.hiddenSelectedTextNode.innerHTML = this.hiddenSelectedText;
		},
		
		deselect: function(){
			this.current = false;
			dojo.removeClass(this.domNode,this.selectedClass);
			this.hiddenSelectedTextNode.innerHTML = this.hiddenUnselectedText;
		},
		destroy: function () {
			this.addButton.onclick = null;
			this.inherited(arguments);
		}
	});
	

dojo.require("com.ibm.pb.customize.LotusDialog");
dojo.require("com.ibm.pb.customize.LotusWaitingDialog");
