/* Copyright IBM Corp. 2010 All Rights Reserved. 
 * @author Laura
 * Summary: Shelf for Page Builder 2
 */

dojo.declare("com.ibm.mm.builder.systemwidget.customizeShelf", [com.ibm.pb.iwidget.iWidgetBase], {
	contextMenuID: "shelfActions",
	menuFile: "shelfActions.json",
	menuModule: "com.ibm.themes.PageBuilder2.menuDefinitions",
	maximized: false,
	loaded: false,
	_someObjectInfo: {
		defaultStyle:"initial",
		defaultLayout:"2ColumnEqual",
		styleMetadataKey: "colorPalette",
		layoutMetadataKey:"com.ibm.portal.layout.template.ref",
		layoutFileMetadataKey:"com.ibm.portal.static.page.file.name.html",
		maxLabelLength: 16,
		maxLabelLengthLarge: 11,
		addContent: ibmCfg.themeConfig.themeRootURI+"/system/addContent.json",
		addContentWCM: ibmCfg.themeConfig.themeRootURI+"/system/addContent_wcm.json",
		addContentAllowed: ibmCfg.themeConfig.themeRootURI+"/system/addContent_allowedList.json",
		changeStyle: ibmCfg.themeConfig.themeRootURI + "/system/changeStyle.json",
		changeLayout: ibmCfg.themeConfig.themeRootURI + "/system/changeLayout.json",
		imagePath: ibmCfg.themeConfig.themeRootURI + "/css/images"
	},
	
	onLoad: function(){
	},
	
	initialize: function(){
		// summary: create shelf and consume the tabs from the context menu API
		this.navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		this.widgetModel = com.ibm.mashups.enabler.widget.Factory.getWidgetModel();
		var pageId = this.runtimeModel.getCurrentPage().getID(),
			page = this.navModel.find(pageId).start(),
			content = page.getContent().start();
		this.contentId = content.getID();
		
		// create context menu and subscribe to events for updates
		var addContentActionsMenu = new com.ibm.pb.contextMenu.JsonContextMenuLoader({url:dojo.moduleUrl(this.menuModule,this.menuFile), data:null});
		this.builderFactory.getContextMenuModel().registerContextMenu(this.contextMenuID, addContentActionsMenu);
		this.evtSvc.subscribeEvent(this.iWidgetConstants.CHANGE_WIDGETWINDOWSTATE, this, this.onWidgetWindowStateChange);
		this.evtSvc.subscribeEvent(this.iWidgetConstants.UNLOAD_WIDGETS, this, this.onUnloadWidgets);

		this.shelf = new com.ibm.pb.customize.enabler.TabbedController({
			childrenArray: this._getData(pageId, dojo.query(".ibm-maximized","layoutContainers").length != 0),
			imagePath: this._someObjectInfo.imagePath,
			navModel: this.navModel,
			widgetModel: this.widgetModel
		}, "customizeShelfContainer");
	},
	
	/* CONTEXT MENU MODEL FUNCTIONS */
	
	_getData: function(pageId, isMaximized){
		// summary: retrieves the controllers from the context menu API
		// pageId: the id of the page from which to get the data
		// isMaximized?: true if a widget was just maximized
		this.maximized = isMaximized;
		var contentIdChanged = this._contentIdChanged();
		var aggStyle = ibmCfg.portalConfig.aggregatedStyle;
		var style = aggStyle && aggStyle != "null" ? aggStyle.substring(aggStyle.lastIndexOf("/")+1) : this._someObjectInfo.defaultStyle;
		var _someObject = dojo.mixin({}, this._someObjectInfo, {isMaximized:isMaximized, contentIdChanged:contentIdChanged, pageDefaultStyle:style});
		var data = this.builderFactory.getContextMenuModel().getContextMenu(this.contextMenuID, pageId, "com.ibm.mm.builder.systemwidget.customizeShelf", _someObject);
		var controllers = [];
		for(var i in data){
			if(data[i].enabled){
				var controller = data[i].actionCallBackFunction();
				controllers.push({controller:controller});
			}
		}
		return controllers;
	},
	
	_refreshTabs: function(pageId, isMaximized){
		// summary: refreshes the controllers from the context menu API and updates the UI
		// pageId: the id of the page for which to refresh
		// isMaximized?: true if a widget was just maximized
		this.shelf.refreshTabs(this._getData(pageId, isMaximized));
	},
	
	_contentIdChanged: function(){
		// summary: returns true if the content id of the page has changed
		var id = this.runtimeModel.getCurrentPage().getID();
		var navNode = this.navModel.find(id).start();
		if(navNode && navNode.getContent) {
			var content = navNode.getContent().start();
			if(content) {
				id = content.getID();
			}
		}
		if(this.contentId == id) { 
			return false;
		} else {
			this.contentId = id;
			return true;
		}
	},

	/* IEVENT HANDLER FUNCTIONS */	
	
	onPageModeChanged: function(iEvent){
		// summary: if the page goes into view mode, make sure the shelf is closed
		var mode = iEvent.payload;
		if (mode == "edit" && !this.loaded) { this.initialize(); this.loaded = true; }
	    if (mode == "view" && this.shelf.displayState == "open") { this.shelf.toggleDisplay(); }
	},
	
	onSavePage: function(iEvent){
		// summary: saves the shelf pending changes
		this.shelf.beforeSave(dojo.hitch(this, function() {
			this.shelf.saveChildren(dojo.hitch(this, function() {
				var pageId = this.runtimeModel.getCurrentPage().getID();
				com.ibm.mashups.services.ServiceManager.getService("eventService").broadcastEvent("com.ibm.mashups.builder.pageSaved",pageId);
			}));
		}));
	},
	
	onPageSwitched: function(iEvent){
		// summary: close the shelf when the page is switched
		var pageId = iEvent.payload;
		this.shelf.clearSelectedTab();
		if(this.shelf.displayState == "open") { this.shelf.toggleDisplay(); }
    },
	
	onLayoutRefreshed: function(iEvent){
		// summary: refresh the tabs when the layout changes
		var pageId = this.runtimeModel.getCurrentPage().getID();
		this._refreshTabs(pageId, dojo.query(".ibm-maximized","layoutContainers").length != 0);
	},
	
	onWidgetWindowStateChange: function(iEvent){
		// summary: handles when a widget or portlet is maximized on a CSA page
		var max = com.ibm.mm.iwidget.Constants.windowstate.MAXIMIZE;
		if(iEvent.newWindowState == max || iEvent.oldWindowState == max) {
			this._refreshTabs(this.runtimeModel.getCurrentPage().getID(), iEvent.newWindowState == max); 
		}
	},
	
	onUnloadWidgets: function(iEvent){
		if (this.maximized) {
			this.maximized = false;
			this._refreshTabs(this.runtimeModel.getCurrentPage().getID(), false); 
		}
	},
		
	onUnload: function(){
		// summary: destroys the shelf on unload of widget
	    if (this.shelf) {
			this.shelf.destroy();
	        this.shelf = null;
	    } 
	}
});
