/* Copyright IBM Corp. 2015  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.DeferredOperation");

dojo.require("com.ibm.mashups.enabler.Deferred");
//<INTERFACE>
/**
 * In addition to the Deferred interface, which allows to trigger callbacks after the
 * entire action  has finished, the DeferredOperation interface allows to trigger a
 * callback for each involved operation.
 * @ibm-api
 * @ibm-module Base
 */
dojo.declare( "com.ibm.mashups.enabler.DeferredOperation", [com.ibm.mashups.enabler.Deferred], {

	/**
	 * Mode of operation: retrieving a resource 
	 * @type String
	 */
    OPERATION_GET: "GET",

	/**
	 * Mode of operation: creating a resource 
	 * @type String
	 */
    OPERATION_CREATE: "CREATE",

	/**
	 * Mode of operation: modifying a resource 
	 * @type String
	 */
    OPERATION_MODIFY: "MODIFY",

	/**
	 * Mode of operation: deleting a resource 
	 * @type String
	 */
    OPERATION_DELETE: "DELETE",

    /**
     * Sets the handler of the deferred action. It is called when the entire
     * action has finished.<br>
     * <b>Notes:</b>
     * <ul> 
     * <li>The status of the action is transported only with callbacks,
     * even if the deferred action is invoked synchronously. If you need to check the status, 
     * you must use callback functions.</li> 
     * <li>The callback function is invoked either in verbose, or non-verbose
     * mode, depending on the verbose flag you set.</li>
     * </ul>
     * @param {Object} callback the callback function in either one of the following flavors: 
     * <ul>
     * <li><b>verbose</b> mode: <code>function(Object verboseFinishResult)</code>, where:<br>
     * <code>verboseFinishResult = {</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>resource:</code> resource object or id string of the related resource,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>statusCode:</code> overall HTTP status code,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>parameter:</code> the parameter passed into <code>setFinishedCallback</code>,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>operations:</code> array of verboseOperationResult, see <code>setOperationCallback</code><br>
     * <code>}</code>
     * </li>
     * <li><b>non-verbose</b> mode: <code>function(Object resource, int statusCode, Object parameter)</code>, where:
     * <ul>
     * <li><code>resource</code> - resource object or id string of the related resource,</li>
     * <li><code>statusCode</code> - overall HTTP status code,</li>
     * <li><code>parameter</code> - the parameter passed into <code>setFinishedCallback</code></li>
     * </ul>
     * </li>
     * </ul>
     * Note that the overall HTTP status code of both the verbose and the non-verbose finished callback
     * is the highest status code of all involved operations.
     * @param {Object} parameter optional parameter to be passed on to the callback function; may be <code>null</code>
     * @param {Boolean} verbose flag to indicate if the callback function is invoked in verbose or non-verbose mode; optional, defaults to false
     * @return {com.ibm.mashups.enabler.Deferred} the deferred object
     */
    setFinishedCallback: function(callback, parameter, verbose){
    },
    
    /**
     * Sets the operation handler of the deferred action. It is called every time an 
     * operation of the action has finished.<br>
     * <b>Notes:</b>
     * <ul> 
     * <li>The status of the action is transported only with callbacks,
     * even if the deferred action is invoked synchronously. If you need to check the status, 
     * you must use callback functions.</li> 
     * <li>The callback function is invoked either in verbose, or non-verbose
     * mode, depending on the verbose flag you set.</li>
     * </ul>
     * @param {Object} callback the callback function in either one of the following flavors: 
     * <ul>
     * <li><b>verbose</b> mode: <code>function(Object verboseOperationResult)</code>, where<br>
     * <code>verboseOperationResult = {</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>resource:</code> resource object or id string of the related resource,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>parameter:</code> the parameter passed into <code>setOperationCallback</code>,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>mode: OPERATION_GET</code> &#124; <code>OPERATION_CREATE</code> &#124; <code>OPERATION_MODIFIY</code> &#124; <code>OPERATION_DELETE</code>,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>request: {</code><br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>statusCode:</code> HTTP status code,<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>responseText:</code> server response text<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;<code>}</code><br>
     * <code>}</code>
     * </li>
     * <li><b>non-verbose</b> mode: <code>function(Object resource, String mode, int statusCode, Object parameter)</code>, where
     * <ul>
     * <li><code>resource:</code> resource object or id string of the related resource,</li>
     * <li><code>mode: OPERATION_GET</code> &#124; <code>OPERATION_CREATE</code> &#124; <code>OPERATION_MODIFIY</code> &#124; <code>OPERATION_DELETE</code>,</li>
     * <li><code>statusCode:</code> HTTP status code,</li>
     * <li><code>parameter:</code> the parameter passed into <code>setOperationCallback</code></li>
     * </ul>
     * </li>
     * </ul>
     * @param {Object} parameter optional parameter to be passed on to the callback function; may be <code>null</code>
     * @param {Boolean} verbose flag to indicate if the callback function is invoked in verbose or non-verbose mode; optional, defaults to false
     * @return {com.ibm.mashups.enabler.Deferred} the deferred object
     */
    setOperationCallback: function(callback, parameter, verbose) {
    }
});

com.ibm.mashups.enabler.DeferredOperation.OPERATION_GET = "GET";
com.ibm.mashups.enabler.DeferredOperation.OPERATION_CREATE = "CREATE";
com.ibm.mashups.enabler.DeferredOperation.OPERATION_MODIFY = "MODIFY";
com.ibm.mashups.enabler.DeferredOperation.OPERATION_DELETE = "DELETE";

//</INTERFACE>

dojo.require( "com.ibm.mm.enabler.DeferredOperationImpl");
