/* Copyright IBM Corp. 2015  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.model.url.ModelUrlFactory");

dojo.require("com.ibm.mashups.enabler.model.url.ModelUrl");
//<INTERFACE>
/**
 * Interface for a ModelUrl Factory that allows to generate ModelUrl's a comfortable way.
 * This class should not be instantiated directly, but should be used through it's defined singleton 
 * (<code>com.ibm.mashups.enabler.model.url.ModelUrlFactory</code>). 
 * @ibm-spi
 * @ibm-module Base
 */
dojo.declare( "com.ibm.mashups.enabler.model.url.ModelUrlFactory", null,
    {
        /**
        * Constant for creation of a navigation URL
        * @type {String}
        */
        NAVIGATION_URL:            "nav",
        
        /**
        * Constant for creation of a layout URL
        * @type {String}
        */
        LAYOUT_URL:                "layout",
                
        /**
        * Constant for creation of a shared navigation URL
        * @type {String}
        */
        SHARED_NAVIGATION_URL:     "sharednav",
        /**
        * Constant for creation of a fragment URL
        * @type {String}
        */
        FRAGMENT_URL:       "fragment",
        /**
        * Constant for creation of a fragment-media URL
        * @type {String}
        */
        FRAGMENT_MEDIA_URL: "fragment-media",
        /**
        * Constant for creation of a user URL
        * @type {String}
        */
        USER_URL:           "user",
        /**
        * Constant for creation of a theme URL
        * @type {String}
        */
        THEME_URL:          "theme",
        /**
        * Constant for creation of a theme-media URL
        * @type {String}
        */
        THEME_MEDIA_URL:    "theme-media",
        /**
        * Constant for creation of a catalog URL
        * @type {String}
        */
        CATALOG_URL:        "cat",
        /**
        * Constant for creation of a config URL
        * @type {String}
        */
        CONFIG_URL:        "config",
        /**
        * Constant for creation of a space URL
        * @type {String}
        */
        SPACE_URL:      "space",
        		/**
        * Constant for creation of a space-favorite URL
        * @type {String}
        */
        SPACE_FAVORITE_URL:      "space-favorite",
        /**
        * Constant for creation of a ac URL
        * @type {String}
        */
        AC_URL:      "ac",
        /**
        * Constant for creation of a ac URL
        * @type {String}
        */
        AI_URL:      "ai",
        /**
        * Constant for creation of a template URL
        * @type {String}
        */
        TEMPLATE_URL:      "template",
        /**
         * Constant for widget model
         * @type {String}
         */
        WIDGET_URL:        "widget",
        /**
         * Constant for event model
         * @type {String}
         */
        EVENT_URL:         "event",
        /**
        * Constant for member sub model
        * @type {String}
        */
        SUBMODEL_MEMBER:    "member",
        /**
        * Constant for role sub model
        * @type {String}
        */
        SUBMODEL_ROLE:      "role",
        
        /**
        * Constant for allowed-access sub model
        * @type {String}
        */
        SUBMODEL_ACCESS:      "access",

        /**
        * Constant for resource sub model
        * @type {String}
        */
        SUBMODEL_RESOURCE:      "resource",
        
        RESOURCE_URL: "resource-service",
                
        /**
        * @private
        **/
        constructor:function () {
        },
        
        /**
        * Create a ModelUrl based on the specified urlType. Afterwards parameters and scheme 
        * specific parts can be changed.<br />
        * Sample:
        * <pre>var myURL = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.NAVIGATION_URL, this);</pre>
        * @param {String} urlType the type of ModelUrl that should be created. You should use the URL constants that are defined in this interface. (Must not be <code>null</code>)
        * @param {com.ibm.mashups.enabler.model.Model} model the model used to create this ModelUrl object for. (Might be <code>null</code>)
        * @type {com.ibm.mashups.enabler.model.url.ModelUrl}
        * @return the created <code>ModelUrl</code>. Returns <code>null</code> if the specified urlType is not known.
        * @deprecated Use createModelURL instead.
        */
        createModelUrl: function (urlType, model) {
        	return new com.ibm.mashups.enabler.model.url.ModelUrl(urlType, model);
        },
        /**
        * Create a ModelUrl based on the specified urlType. Afterwards parameters and scheme 
        * specific parts can be changed.<br />
        * Sample:
        * <pre>var myURL = com.ibm.mashups.enabler.model.url.ModelUrlFactory.createModelURL(com.ibm.mashups.enabler.model.url.ModelUrlFactory.NAVIGATION_URL, this);</pre>
        * @param {String} urlType the type of ModelUrl that should be created. You should use the URL constants that are defined in this interface. (Must not be <code>null</code>)
        * @param {com.ibm.mashups.enabler.model.Model} model the model used to create this ModelUrl object for. (Might be <code>null</code>)
        * @type {com.ibm.mashups.enabler.model.url.ModelUrl}
        * @return the created <code>ModelUrl</code>. Returns <code>null</code> if the specified urlType is not known.
        */
        createModelURL: function (urlType, model) {
        	return new com.ibm.mashups.enabler.model.url.ModelUrl(urlType, model);
        },
        /**
        * Create a ModelUrl based on the specified URL. Afterwards parameters and scheme specific parts can be changed.
        * @param {String} url the URL that should be used to create a <code>ModelUrl</code>. (Must not be <code>null</code>)
        * @param {com.ibm.mashups.enabler.model.Model} model the model used to create this ModelUrl object for. (Might be <code>null</code>)
        * @type {com.ibm.mashups.enabler.model.url.ModelUrl}
        * @return the created <code>ModelUrl</code> (Must not be <code>null</code>).
        * @deprecated Use getModelURL instead.
        */
        getModelUrl: function (url, model) {
        	return new com.ibm.mashups.enabler.model.url.ModelUrl(url, model);
        },
        /**
        * Create a ModelUrl based on the specified URL. Afterwards parameters and scheme specific parts can be changed.
        * @param {String} url the URL that should be used to create a <code>ModelUrl</code>. (Must not be <code>null</code>)
        * @param {com.ibm.mashups.enabler.model.Model} model the model used to create this ModelUrl object for. (Might be <code>null</code>)
        * @type {com.ibm.mashups.enabler.model.url.ModelUrl}
        * @return the created <code>ModelUrl</code> (Must not be <code>null</code>). 
        */
        getModelURL: function (url, model) {
        	return new com.ibm.mashups.enabler.model.url.ModelUrl(url, model);
        }
    }
);
//</INTERFACE>

dojo.require("com.ibm.mm.enabler.model.url.ModelUrlFactoryImpl");
