/* Copyright IBM Corp. 2015  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewer");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.string");
dojo.require("dijit.TitlePane");

dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataFilter");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.VerticalDataViewer");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColumnsList");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColorPickerButton");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataSource");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.DataViewerException");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ColumnManager");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.EmptyTable");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.ViewConfig");

/*
 * Data Viewer is used to display tabular data. It supports two data sources 1) hub data feed 2) tabular data from other iwidget via wiring
 * For data mismatch, there are two solutions. 1) Leverage hub capability to support CSV, Excel, Access etc. 2) Leverage transformation iWidget
 * to transform non-tabular data to tabular data (JSON array)
 * Data Viewer supports end user to customize the data display setting.
 * 
 * Method onLoad, onview and onedit are iWidget interface.
 * Method handleJSONArray, handleURL, onDataSelected are for iWidget wiring.
 */

/*
 * From design point of view, com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewer implement elements required by iWidget SPEC and 
 * glues below classes to work together to accomplish data viewer's functionality.
 * 
 * Data source take the responsibility of retrieving data from remote URL, dojo data store and other wired widgets
 * Data model take the responsibility of interpreting the data from data source. JSON Array, XML document, CSV document
 * Entry data model take the responsibility of interpreting the ATOM entry data.
 * com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer take the responsibility of rendering the data from data source based on 
 * rendering configuration from iWidget itemset.
 * com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil, com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColorPalette and com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColumnsList
 * are utility function or classes.
 * 
 * If the future change is to support new data source. You need to write new data source, data model or entry data model.
 * If the future change is to support new rendering configuration, you need to enhance com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer
 *  
 */
dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewer", null, {
	
	// Constant
	INITIAL_TABLE_HTML: "<div><table class='mumDataViewer-_WB_VER_-v210 default' role='presentation'><thead><tr><th scope='col'><span>${L_COLUMN}1</span></th><th scope='col'><span>${L_COLUMN}2</span></th><th scope='col'><span>${L_COLUMN}3</span></th><th scope='col'><span>${L_COLUMN}4</span></th><th scope='col'><span>${L_COLUMN}5</span></th></tr></thead><tbody><tr><td><span>${L_ROW}1</span></td><td/><td/><td/><td/></tr><tr class='even'><td><span>${L_ROW}2</span></td><td/><td/><td/><td/></tr></tbody></table></div>",
	INITIAL_TABLE_FOOTER: "<div class='pageIndex default'>${L_TABLE_PAGE}&nbsp;1</div><div class='pageBtn default'>${L_TABLE_PREVIOUS} <span class='pageLinkSeparator'>|</span> ${L_TABLE_NEXT}</div>",

	ID_DIJIT_DIV: 'urlDijitDiv',       // dom id for url.dijit
    ID_VURLOBJ: "vURL",
		
	reload: true,
    viewInitiated:false,
	columnManager: null,
	
	_urlIsFromWiring: false,
	_urlFromWiring: null,
	_editNodeInnerHTMLCache: null,//object must be initialized in constructor, instead of here
    _viewNodeInnerHTMLCache: null,
	_onChangeUrlConnect: null,
	_eventConnects: null,
	_dataSource: null,
    _wires: null,
	wiringCount: 0,
	
	// URL Dijit parameters
	_urlDijitClass: "com.ibm.mashups.widget.dijit.url.v104",
    _urlWidgetHelper: null,
    _urlDijit: null,
    _dataReceivedLocally: false,
    
	constructor: function(){
		this.hasData = false;
		this.dynamicEvents = {};
		this.domID = null;
		this.dataModels = null;
		this.message = "";
		this.resourceBundle = null;
		this._eventConnects = [];
        this._wires = {};
		this.publishEventEnabled = {
			'New_Feed_Address' : true,
			'New_CSV_Address' : true,
			'Send_Table' : false,		// performance optimizing, do not general table data if no wiring exists.
			'Row_Selection' : true,
			'Send_Row_as_Table' : true,
			'Select_Cell_as_Table' : true,
			'Select_Cell_as_Text' : true,
			'Dynamic_Event' : {}
		};
		this._editNodeInnerHTMLCache={};
		this.LOGGER = com.ibm.mashups.enabler.logging.Logger.getLogger("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewer");
	},
    
    getMessages: function(cs){
        try {
            dojo['requireLocalization']("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer", cs.getValue("locale"), cs.getValue("availableLocales"));
            var msg = dojo.i18n.getLocalization("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer");
            dojo['requireLocalization']("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer", cs.getValue("default.locale"), cs.getValue("availableLocales"));
             var defaultMsg = dojo.i18n.getLocalization("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer",cs.getValue("default.locale"));
             return dojo.mixin(defaultMsg,msg);
        } 
        catch (e) {
            // fallback to default.locale to be able to tolerate non-supported languages
            return this.getDefaultMessages(cs);
        }
    },

    getDefaultMessages: function(cs){
        dojo['requireLocalization']("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer", cs.getValue("default.locale"), cs.getValue("availableLocales"));
        return dojo.i18n.getLocalization("com.ibm.mm.widget.dataViewer._WB_VER_.v210", "dataViewer");
    },
	/*
	 * Invoked when data viewer is loaded
	 */
	onLoad: function(){
		var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		
		dojo.registerModulePath("com.ibm.mm.widget.dataViewer._WB_VER_.v210",this.iContext.io.rewriteURI("js/com/ibm/mm/widget/dataViewer"));
        
		 com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages=this.getMessages(cs);
		dojo.registerModulePath(this._urlDijitClass, this.iContext.io.rewriteURI("urlDijit"));
		dojo["require"](this._urlDijitClass + "._base");
		
        var att=this.iContext.getiWidgetAttributes();
        this._viewConfig = this._loadItemSet();
        var url = this._viewConfig.get("feedURL");
        
		var hubUrl = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME).getValue(com.ibm.mashups.enabler.services.ConfigConstants.HUB_URL);
		
		// instantiate the Widget Helper
        this._urlWidgetHelper = new (dojo.getObject(this._urlDijitClass).WidgetHelper)({
			migrateUrl: url || "",
            iContext: this.iContext,
            openSearchUrl: hubUrl + '/atom?nologin=true',
            attributeKey: this.ID_VURLOBJ,
			hideLoadButton: true,
			hideBrowseButton: (!hubUrl),
			autoSave: false,
			noForm: true
        });
        
        this.contextRoot = this.iContext.io.rewriteURI("");
        this.resourceBundle = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages;
		this.reload = true;

		this.domID = "_" + this.iContext.widgetId + "_";
		
		this.columnManager = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.ColumnManager();
		
		this.currentSource = {};
		
		this._subscribe = dojo.subscribe("dataViewer/" + this.iContext.widgetId + "/dataSelected", this, "onDataSelected");
	},
	
	onUnload: function(){
		dojo.unsubscribe(this._subscribe);

		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer && dataViewer.destroy) {
			dataViewer.destroy();
		}

		if (this.ColumnTable && this.ColumnTable.destroy){
			this.ColumnTable.destroy();
		}
        
        this.destroyUrlDijit();
        this.destroyTempDijits();
		dojo.forEach(this._eventConnects, dojo.disconnect);
	},
	
    destroyUrlDijit:function (){
         if (this._urlDijit && this._urlDijit.destroyRecursive) {
            this._urlDijit.destroyRecursive(true);
            delete this._urlDijit;
            this._urlDijit = null;
        }

    },
    
    destroyTempDijits:function(){
        if (this._tempWidgets) {
            // Destroy dijits in edit mode so we can instantiate them when going to edit mode again
            try {
                if (dojo.isArray(this._tempWidgets)) {
                    dojo.forEach(this._tempWidgets, function(widget){
                        if (widget) {
                            if (widget.destroyRecursive) {
                                widget.destroyRecursive();
                            }
                            else 
                                if (widget.destroy) {
                                    widget.destroy();
                                }
                        }
                    });
                }
                delete this._tempWidgets;
                this._tempWidgets = null;
            } 
            catch (e) {
                // Prevent "Permission denied" exception in sandbox. #10941
                console.error(e);
            }
        }
    },
	/*
	 * Invoked when data viewer to display view mode
	 */
	onview: function(){
        this.viewInitiated=true;
		var att = this.iContext.getiWidgetAttributes();
		if (this.reload) {
			var viewRootNode = this.byId("dataViewerViewRoot");
            // Apply NLS
            this._viewNodeInnerHTMLCache = this._viewNodeInnerHTMLCache || dojo.trim(dojo.string.substitute(viewRootNode.innerHTML, this.resourceBundle));
            dojo.place(this._viewNodeInnerHTMLCache, viewRootNode, "only");
			
			this._viewConfig = this._loadItemSet();
			
            //this.updateWiringFlag(); 20767: it is too early to set this.  If you set it now it will always be false.
			//var feedURL = this._viewConfig.get('feedURL'); //defect 18600
			/*
			prefer to get the URL from the 'feedURL' item set to comply with the 
			peronalize > eidt > config mode.
			If it's blank, in the case of migation, we get it from URL dijit.
			*/ 
			var feedURL = att.getItemValue("feedURL");
			if(!feedURL || feedURL.length==0) {
				feedURL = this._urlWidgetHelper.getUrl();
			}
			this._viewConfig.set('feedURL', feedURL);
			var isWiring = this._viewConfig.get('isWiring');
			if (!feedURL && !isWiring) {
				this.showInitMessage();
			} else {
				if (isWiring && this._urlIsFromWiring) {
					// If url is from eventing, set the url to eventing payload url
					this._viewConfig.set('feedURL', this._urlFromWiring);
				}
			    this.renderDataFromURL(this._viewConfig);
			}
			
			// Scrollbar is needed for data viewer
			// 20381, disalbe overflow since IE doesn't like it while page switch. Scrollbar is handled by skin. 
//			dojo.style(this.iContext.getRootElement(), "overflow", "auto");
			
	        if (this._viewConfig) {
	            delete this._viewConfig;
	            this._viewConfig = null;
	        }
		}
		
        if (this._urlDijit && this._urlDijit.destroyRecursive) {
            this._urlDijit.destroyRecursive(true);
            delete this._urlDijit;
            this._urlDijit = null;
        }
		dojo.disconnect(this._onChangeUrlConnect);
        this._onChangeUrlConnect = dojo.connect(this._urlWidgetHelper, "onUpdateUrl", this, "_onUpdateUrl");
		
		dojo.disconnect(this._contentTypeHandler);
	},
    
    setPreTitle:function(divId,title){
        var div = this.byId(divId);
        if(!!div){
            div.title = title;
        }
    },
	
	/*
	 * Invoked when data viewer to display config view
	 */
	onedit: function(){
		var configRootNode = this.byId("dataViewerConfigRoot");
        // Apply NLS
        this.setPreTitle("DATA_PANE",this.resourceBundle.L_CONF_DATA_SELECTION_LINK);
        this.setPreTitle("TABLE_PANE",this.resourceBundle.L_CONF_CONTENT_LINK);
        this.setPreTitle("APPEARANCE_PANE",this.resourceBundle.L_CONF_STYLE_LINK);
        this.setPreTitle("FILTER_PANE",this.resourceBundle.L_CONF_FILTER_LINK);
        var idSuffix = this.getIdSuffix();
        if(!idSuffix) idSuffix="";
        this._editNodeInnerHTMLCache[idSuffix] = this._editNodeInnerHTMLCache[idSuffix] || dojo.trim(dojo.string.substitute( configRootNode.innerHTML, this.resourceBundle));
        dojo.place(this._editNodeInnerHTMLCache[idSuffix], configRootNode, "only");
		this._fixImagePath(configRootNode);
        
        this.destroyTempDijits();
		
		this._tempWidgets = dojo.parser.parse(configRootNode);
		
		this.updateWiringFlag();

		var settings = this._loadItemSet();
        var columnSettings = settings.getColumnSettings(this.currentSource);
		if (columnSettings) {
			try{
				if (this.ColumnTable){
					this.ColumnTable.destroy();
					this.ColumnTable = null;
				}
				this.ColumnTable = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColumnsList({
					wid: this.domID + "field",
					contextRootBuilder: this.contextRoot
				}, this.byId("ColumnTable"));
				
				this.ColumnTable.setSettings(columnSettings);
				this.ColumnTable.render();
			}catch(e1){
				this.LOGGER.severe("onedit", this.resourceBundle.E_ITEMSET_COLUMNS_BROKEN);
			}
		}
        settings.renderEditView(dojo.hitch(this,this.byId),dojo.hitch(this,this.dijitById));

        var contentTypeSelect = this.byId("contentType");
	     var contentCompareSelect = this.byId("contentCompare");
		this.changeCompare(contentTypeSelect.value, contentCompareSelect, settings.get('filterFunction'));
		
		this._contentTypeHandler = dojo.connect(contentTypeSelect, "onchange", this, function(){
			this.changeCompare(contentTypeSelect.value, contentCompareSelect);
		});
        
		this._urlDijit = this._urlDijit || this._urlWidgetHelper.initializeDijit(this.byId(this.ID_DIJIT_DIV));
        dojo.disconnect(this._onChangeUrlConnect);
        if (settings.get('isWiring')){
            this.showMessage(this.resourceBundle.L_CONF_WIRING, "warning", "wiring");
			this._disableUrlDijit(true);
        } else{
            if(!!settings.get(this.ID_VURLOBJ)){
                if (this._urlWidgetHelper.vUrl.serialize() != settings.get(this.ID_VURLOBJ)) {
                    this._urlWidgetHelper.vUrl.deserialize(settings.get(this.ID_VURLOBJ));
                }
            }else{
                if (this._urlWidgetHelper.getUrl() != settings.get('feedURL')){
                    this._urlWidgetHelper.setUrl(settings.get('feedURL'));
                }
            }
            this.showMessage(false, null, "wiring");
			this._disableUrlDijit(false);
        }
		this._onChangeUrlConnect = dojo.connect(this._urlWidgetHelper, "onUpdateUrl", this, "onLoadButtonClick");

		//20890 The next 'if' will be true if the URL came in from a wire
        if(settings!=null && settings.get('feedURL') == null && (settings.get('columns')==null || dojo.toJson(settings.get('columns')) =="{}")){
        	var urlWidgetHelperUrl = this._urlWidgetHelper.getUrl();
        	if (urlWidgetHelperUrl) {
        		this.onLoadButtonClick(urlWidgetHelperUrl);
        	}
        }	
	},
    
    onConfig: function() {
      this.onedit();
    },
  
    onPersonalize: function() {
   	  this.onedit();
    },
	
    /**
     * Change the path of image node in rootNode to canonical path
     * @param {domNode} root dom node
     */
    _fixImagePath: function(rootNode) {
        if (rootNode) {
            var baseURL = this.iContext.io.rewriteURI(".");
            dojo.query("img[src],input[src]", rootNode).forEach(function(imgNode){
                var src = dojo.attr(imgNode, "src");
                var canonicalSrc = src.replace(/\S*_ROOT_/, baseURL);
                dojo.attr(imgNode, "src", canonicalSrc);
            }, this);
        }
    },

    dijitById: function(id){
        return dijit.byId(id+this.getIdSuffix());
    },
    /**
     * Get dom node by id in iWidget
     * @param {String|DomNode} id
     */
    byId: function(id) {
        if(!id) return null;
        return this.iContext.getElementById(this.domID + id+this.getIdSuffix()) || this.iContext.getElementById(id+this.getIdSuffix());
    },
    getIdSuffix: function(){
        var currentModeSuffix = this.iContext.widgetwrapper.currentMode;
        if (!currentModeSuffix || currentModeSuffix == "view") {
            return "";
        }else {
            return "_"+currentModeSuffix;
        }
    },

	_disableUrlDijit: function(disabled) {
	    this._urlWidgetHelper.disableUrlInput(disabled);
        this._urlWidgetHelper.disableLoadButton(disabled);
	},
	
	_getValueFromJson: function(json, attribute, defaultValue){
		try {
			var obj = dojo.fromJson(json);
			return obj[attribute];
		}catch(e){
			return defaultValue;
		}		
	},
	
	_handleReceivedData: function(iEvent){
        this._dataReceivedLocally=true;
		this.currentSource = {
			source: iEvent.source,
			name: iEvent.name
		};
        var sourceId = iEvent.source;
		var self = this;
        if (!this._wires[sourceId]) {
            this._wires[sourceId] = true;
            ++this.wiringCount;        // enable wired mode, can not change url
            this.updateWiringFlag();   // 20767: need to change flag when count changes
        }
		//this._urlIsFromWiring = false; // 20767: this is set by updateWiringFlag
		this._dataSource = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.localDataSource(iEvent.payload, function(e){
			self.showErrorMessage(e);
		});
		this._dataSource.getData(function(data){
			this._viewConfig = self._loadItemSet();
			self.processData(data, this._viewConfig);
			self.setWiringFlag(true);

			self.publishTable(data);
		});
	},

	/*
	 * This method supports display data from other iWidget, payload type is JSON array
	 */
	handleJSONArray: function(iEvent){
		if (iEvent && dojo.isString(iEvent.payload)) {
			iEvent.payload = dojo.fromJson(iEvent.payload);
		}
		this._handleReceivedData(iEvent);
	},

	/*
	 * This method supports display data from other iWidget, payload type is ATOM feed XML document 
	 */
	handleXMLData: function(iEvent){
		this._handleReceivedData(iEvent);
	},
	_onUpdateUrl: function(dataSourceURL) {
		this._urlParameterChanged = true;
		this._onChangeUrl(dataSourceURL);
	},
	
    _onChangeUrl: function(dataSourceURL) {
		if(this.urlOnChangeOnGoingMarker){
			setTimeout(dojo.hitch(this, function(){
                this._onChangeUrl(dataSourceURL);
            }), 5);
			return;
		}
		this.urlOnChangeOnGoingMarker=true;
        this._viewConfig = this._loadItemSet();
        this._viewConfig.set('feedURL', dataSourceURL);
        this.renderDataFromURL(this._viewConfig);        
    },
    
	/*
	 * This method supports display data from other iWidget, payload is resource url
	 */
	handleURL: function(iEvent){
		var url = iEvent.payload,
            sourceId = iEvent.source;
		
		url = dojo.trim(url);
		if (!url) return;
		
		this.currentSource = {
			source: url,
			name: iEvent.name
		};
		
		if (!this._wires[sourceId]) {
            this._wires[sourceId] = true;
            ++this.wiringCount;        // enable wired mode, can not change url
            this.updateWiringFlag();   // 20767: need to change flag when count changes
        }
		//this._urlIsFromWiring = true;  // 20767: this gets set by updateWiringFlag
		this._urlFromWiring = url;
        this._urlWidgetHelper.setUrl(url);
	},
	
	/*
	 * This method supports display data from other iWidget, payload is Dojo data store
	 */
	handleDataStore: function(iEvent){
        this._dataReceivedLocally=true;
		var self = this;
		var store = iEvent.payload;
        var sourceId = iEvent.source;
		
		this.currentSource = {
			source: iEvent.source,
			name: iEvent.name
		};
		
		if(dojo.isString(store)){
			//csv data, for Symphony integration
			store = new com.ibm.mm.data.table({'url': '', 'data': iEvent.payload});
		}
        this._urlWidgetHelper.setUrl("");
		if (!this._wires[sourceId]) {
            this._wires[sourceId] = true;
            ++this.wiringCount;        // enable wired mode, can not change url
            this.updateWiringFlag();   // 20767: need to change flag when count changes
        }
		this._dataSource = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.DojoDataStoreDataSource(store, function(e){
			self.showErrorMessage(e);
		});
		//this._urlIsFromWiring = false; // 20767: this is set by updateWiringFlag
		this._dataSource.getData(function(data){
			this._viewConfig = self._loadItemSet();
			self.processData(data, this._viewConfig);
			self.setWiringFlag(true);

			self.publishTable(data);
		});

	},
	
	handleRemoveWire: function(iEvent) {
		var sourceId = iEvent.payload.sourceWidget;
        if (this._wires[sourceId]) {
            this._wires[sourceId] = false;
            delete this._wires[sourceId];
            --this.wiringCount;
            this.wiringCount = this.wiringCount < 0 ? 0 : this.wiringCount;
            this.updateWiringFlag();   // 20767: need to change flag when count changes
        }
	},
	
	setWiringFlag: function(isWired){
		var att=this.iContext.getiWidgetAttributes();
        if(att.getItemValue("isWiring")+'' != isWired+''){//make sure only the changed value get set
            att.setItemValue("isWiring", isWired, false);
            // 20767: this value needs to be saved here, because edit mode may not be invoked while wiring
            this.reload = true;
            att.save();
        }
        //invoke in both edit/view mode,for view mode, do not invoke save,
        //in edit mode, save will be done when user click save button. 
//		att.save();
          
		if (this._viewConfig) {
			this._viewConfig.set('isWiring', isWired);
		}
	},
	
	updateWiringFlag: function(){
		if (!this.wiringCount){
			this.setWiringFlag(false);
            this._urlIsFromWiring = false;
		} else{
			this.setWiringFlag(true);
			this._urlIsFromWiring = true; //20767: both flags need to be set
		}
	},

	/*
	 * This method supports to publish events when table row is selected.
	 */
	onDataSelected: function(message){
		if (this.publishEventEnabled['Row_Selection']){
	        this.iContext.iEvents.publishEvent("Row Selection", message.dataInfo.rowData);			
		}
        if (this.publishEventEnabled['Send_Row_as_Table']){
            this.iContext.iEvents.publishEvent("Send Row as Table", new com.ibm.mm.data.table({'data': com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.JSONObject2CSV(message.dataInfo.rowJSonData)}));        	
        }
        if (this.publishEventEnabled['Select_Cell_as_Table']){
            this.iContext.iEvents.publishEvent("Select Cell as Table", new com.ibm.mm.data.table({'data': com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.JSONObject2CSV(message.dataInfo.cellJSonData)}));        	
        }
        if (this.publishEventEnabled['Select_Cell_as_Text']){
        	this.iContext.iEvents.publishEvent("Select Cell as Text", message.dataInfo.cellValue);
        }
        
		for (var key in message.dataInfo.rowJSonData){
            if (this.dynamicEvents[key] /*&& this.publishEventEnabled['Dynamic_Event'][key]*/) {
                this.iContext.iEvents.publishEvent(key, message.dataInfo.rowJSonData[key], "string");
            }
		}
	},
	
	publishNewAddressOnNewWire: function(){
		var att=this.iContext.getiWidgetAttributes();
		var feedURL = this._urlWidgetHelper.getUrl();
		var urlType = att.getItemValue("type");
		if (feedURL) {
			// TODO: two events should be merged into one: data source url
			this.publishEventEnabled['New_Feed_Address'] = true;
			this.iContext.iEvents.publishEvent("New Feed Address", feedURL);
			this.publishEventEnabled['New_CSV_Address'] = true;
			this.iContext.iEvents.publishEvent("New CSV Address", feedURL);
		}
	},
	
	publishSendTableOnNewWire: function(){
		this.publishEventEnabled['Send_Table'] = true;	
		//publish table
		if(this.message){
			this.publishTable(this.message);
		}
	},
	
    publishTable: function(data){
		if (this.publishEventEnabled['Send_Table']){
            var csv = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.DataObject2CSV(data);
            var table = new com.ibm.mm.data.table({'url': "", 'data': csv});
            this.iContext.iEvents.publishEvent("Send Table", table);
        }
    },
    
	publishRowSelectionOnNewWire: function(){
		this.publishEventEnabled['Row_Selection'] = true;		
	},
	
	publishSendRowAsTableOnNewWire: function(){
		this.publishEventEnabled['Send_Row_as_Table'] = true;		
	},
	
	publishSelectCellAsTableOnNewWire: function(){
		this.publishEventEnabled['Select_Cell_as_Table'] = true;		
	},
	
	publishSelectCellAsTextOnNewWire: function(){
		this.publishEventEnabled['Select_Cell_as_Text'] = true;		
	},
	
	publishDynamicEventOnNewWire: function(iEvent){
        var payload = iEvent.payload;
        var eventName = payload.sourceEvent;
		this.publishEventEnabled['Dynamic_Event'][eventName] = true;
	},
	
	/*
	 * Below methods are used internally as utility.
	 */
	
	/**
	 * Show messages
	 * @param {String} message
	 * @param {String} type (optional) type can be "info", "error", "warning". if type is null, "error" is default.
	 */
	showMessage: function(message, type, messageBox) {
        messageBox = this.byId(messageBox) || this.byId("messageBox");
		dojo.empty(messageBox);
		
		if (type != "info") { 
			var captionNode = this.byId("caption");
			if (captionNode != null) 
				dojo.empty(captionNode);
		}
		
		if (!message) {
			/*	Performance consideration, disable message fadeout animation
			dojo.fadeOut({ node: messageBox }).play();
			*/
            messageBox.style.display = "none";
            return;
        }
        		
        var img = dojo.create("img");
        var span = dojo.create("span");
        span.appendChild(dojo.doc.createTextNode(message));
        
        switch(type) {
            case "info":
                img.alt = this.resourceBundle.L_INFORMATION;
                img.src = this.iContext.io.rewriteURI("images/info_icon.png");
                dojo.toggleClass(messageBox, "lotusWarning", false);
                dojo.toggleClass(messageBox, "lotusInfo", true);
				dijit.removeWaiRole(messageBox, "alert");
                break;
            case "warning":
                img.alt = this.resourceBundle.L_WARNING;
                img.src = this.iContext.io.rewriteURI("images/warning_icon.png");
                dojo.toggleClass(messageBox, "lotusWarning", true);
                dojo.toggleClass(messageBox, "lotusInfo", false);
				dijit.removeWaiRole(messageBox, "alert");
                break;
            default:       // "error" as default
                img.alt = this.resourceBundle.L_ERROR;
                img.src = this.iContext.io.rewriteURI("images/error_icon.png");
                dojo.toggleClass(messageBox, "lotusWarning", false);
                dojo.toggleClass(messageBox, "lotusInfo", false);
				dijit.setWaiRole(messageBox, "alert");
        }
        messageBox.appendChild(img);
        messageBox.appendChild(span);
        messageBox.style.display = "block";
	},
	
    showInitMessage: function(){
        this.byId("loadingMessage").style.display = "none";
        this.byId("iTableEdit").style.display = "";
        
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
        if (dataViewer) dataViewer.destroy(false);
        
        this.byId("dataViewerId_outerTD").innerHTML = dojo.string.substitute(this.INITIAL_TABLE_HTML, this.resourceBundle);
		this.byId("pageLink").innerHTML = dojo.string.substitute(this.INITIAL_TABLE_FOOTER, this.resourceBundle);
        
		// use "innerHTML" here, because initial text has structure
        this.showMessage(" ", "info");
		dojo.query("span", this.byId("messageBox"))[0].innerHTML = this.resourceBundle.I_INIT_TEXT;
    },
    
	showErrorMessage: function(error, messageBox){
		messageBox = this.byId(messageBox) || this.byId("messageBox");
		if (messageBox === this.byId("messageBox")) {
			this.byId("loadingMessage").style.display = "none";
			this.byId("iTableEdit").style.display = "";
			
			var dataViewer = this.dijitById(this.domID + "dataViewerId");
			if (dataViewer) {
				dataViewer.destroy(false);
			} 
			else {
				// empty initial table, which does not contain a widget
				var outerTd = this.byId("dataViewerId_outerTD");
				dojo.empty(outerTd);
			}
        }
		
		var errorMsg = "", 
			type = "error",
			md = com.ibm.mm.widget.dataViewer._WB_VER_.v210;
		if (!this._viewConfig) {
			this._viewConfig = this._loadItemSet();
		}
		
		var captionNode = this.byId("caption");
		dojo.empty(captionNode);
		dojo.style(captionNode, "display", "none");
		
		if (error instanceof md.NoContentException) {
			if (messageBox === this.byId("messageBox")) {
				var emptyTable = new md.widget.EmptyTable({
					id: this.domID + "dataViewerId"
				}).placeAt(this.byId("dataViewerId_outerTD"));
				
				// Show table caption above info message
				captionNode.innerHTML = this._viewConfig.get('tableTitle');
				dojo.style(captionNode, this._viewConfig.getCaptionNode());
			}
			
			errorMsg = this._viewConfig.get('noContentMessage') || error.message;
			type = "info";
		} else {
			if (messageBox === this.byId("messageBox")) {
				var emptyTable = new md.widget.EmptyTable({
					id: this.domID + "dataViewerId"
				}).placeAt(this.byId("dataViewerId_outerTD"));
			}
			
			if (error instanceof md.IOException) {
				errorMsg = this._viewConfig.get('invalidFeedMessage') || error.message;
			} else if (error instanceof md.DataViewerException){
				errorMsg = error.message;
			} else if (error.constructor.name === TypeError){
				errorMsg = md.dataViewerMessages.E_JS_TYPEERROR;
			} else {
				errorMsg = error;
			}
		}
		this.byId("pageLink").innerHTML = dojo.string.substitute(this.INITIAL_TABLE_FOOTER, this.resourceBundle);
		
		errorMsg =  errorMsg || md.dataViewerMessages.E_UNKNOWN_ERROR;
		this.showMessage(errorMsg, type, messageBox);
	},
	
	showErrorMessageTableExist: function(error){
		this.byId("loadingMessage").style.display = "none";
		this.byId("iTableEdit").style.display = "block";
		this.showMessage(error);
	},
	
	showLoadingMessage: function(){
		this.byId("loadingMessage").style.display = "block";
		this.showMessage(false);
		this.byId("iTableEdit").style.display = "none";
	},
	
	generateDojoDataStore : function(targetData) {
		var json = {"items" : []};
        for (var i = 0; i < targetData.length; i++) {
            var item = {};
            for (var key in targetData[i]) {
            	item[key] = targetData[i][key].toString();
            }
            json.items.push(item);
        }
		var store = new dojo.data.ItemFileReadStore({data : json});
		delete store._features["dojo.data.api.Identity"];
		return store;
	},
	
	/*
	 * Render tabular data to table according to viewConfig settings, set filter rule if necessary
	 */
	renderTable: function(data, viewConfig){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.destroy();
			dataViewer = null;
		}

		var outerTd = this.byId("dataViewerId_outerTD");
		dojo.empty(outerTd);
		var containerDiv = dojo.create('div');
		outerTd.appendChild(containerDiv);
		
        var columnSettings = viewConfig.getColumnSettings(this.currentSource);
		var renderConstructorParams = {
				id: this.domID + "dataViewerId",
				mmWidgetId: this.iContext.widgetId,
				imageBase: this.iContext.io.rewriteURI("images/"),
				dataViewerStyle: viewConfig.getViewerStyle(this.currentSource),
			columnsOrder: columnSettings['columnsOrder']
		};
		
		var rowsLimit = parseInt(viewConfig.get('rowsLimit'), 10);
		var columnFilter = columnSettings['columnFilter'];
		
		if(!viewConfig.get('horizontalView') && ( rowsLimit === -1 || data.totalResult <= rowsLimit)){		
			dataViewer = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.VerticalDataViewer(renderConstructorParams, containerDiv);	
			dataViewer.setRowsPerPage(1);
					
		}else {
			dataViewer = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer(renderConstructorParams, containerDiv);
			var rowsPP = viewConfig.get('rowsPerView');
			dataViewer.setRowsPerPage(parseInt(rowsPP, 10));
		}
		
		var columnLabel = columnSettings['columnLabel'];
		var columnSorttype = columnSettings['columnSorttype'] || "String";
		
		var eventName = {};
		for (var i = 0, length = data.columnList.length; i < length; i++) {
			eventName[data.columnList[i]] = columnLabel[i];
		}
		
		this.updateDynamicEvents(eventName);
		//add columnLabel as input parameter
		dataViewer.fetchAllInfo(data, columnFilter,columnLabel,columnSorttype);
		var ownId = this.domID;
		dataViewer.setID(ownId);
		
		var dataFilter = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataFilter({
			filterColumn: viewConfig.get('filterColumn'),
			filterFunction: viewConfig.get('filterFunction'),
			filterTargetValue: viewConfig.get('filterTargetValue'),
			isFilter: viewConfig.get('isFilter'),
			isHighLightFilter: viewConfig.get('isHighLightFilter'),
			isRemoveFilter: viewConfig.get('isRemoveFilter'),
			columnLabel: columnLabel,
			columnSorttype: columnSorttype,
			data: data
		});
		
		var _store = dataFilter.filterData();
		dataViewer.setStore(_store);		
		dataViewer.renderFirstPage();
		
		if (viewConfig.get("noSort")) {
		    dataViewer.onSort = function() {};
		}
		
		if (dataViewer.getRowsCount() === 0) {
			var emptyMessage = viewConfig.get('allContentFilteredMessage') || this.resourceBundle.I_ALL_CONTENT_FILTERED;
			this.showMessage(emptyMessage, "info");
		} else {
			dojo.query("table.mumDataViewer-_WB_VER_-v210 tr td a[href]", this.byId("dataViewerId_outerTD")).forEach(function(td){
				// Prevent from redirecting page when clicking link in data table
				dojo.attr(td, "target", "_blank");
			});
		}
	},
		
	processData: function(data, viewConfig){
		this.byId("loadingMessage").style.display = "none";

		this.showMessage(false);

		this.message = data;
		
		viewConfig.resetColumnsSetting(data,this.iContext.getiWidgetAttributes());
		
		this.hasData = true;
		this.showTable(data, viewConfig);
	},
	
	
	/*
	 * This method supports table column display order/filter/alignment
	 */
	prepareDataForColumnListWidget: function(columnList){
		if (columnList.length <= 0){
			throw(new com.ibm.mm.widget.dataViewer._WB_VER_.v210.NoContentException(this.resourceBundle.E_NO_CONTENT));
		}

		var columns = [];
		for (var i = 0; i < columnList.length; i++) {
			var name = columnList[i];
			//add label and sorttype to column datatype
			var column = {
				'name': name,
				'align': 'left',	// default
				'checked': true,	// default
				'index': i,
				'label': name,
				'sorttype':'String'				
			};
			columns.push(column);
		}
		return columns;
	},
	
	showTable: function(data, viewConfig){
		if (this.hasData) {
			var tableEdit = this.byId("iTableEdit");
			this.renderTable(data, viewConfig);
			this.updatePageLink();
		// hide page links if need
		this.hidePageLinkIfNeedWhenVerticalView(data, viewConfig);

			tableEdit.style.display = "";
		}
	},
	
	hidePageLinkIfNeedWhenVerticalView: function(data, viewConfig){
		
		if(!viewConfig.get('horizontalView') && data.totalResult === 1 ){
		
			var rowsLimit = parseInt(viewConfig.get('rowsLimit'), 10);
			var linktd = this.byId("pageLink");
			
			if(rowsLimit === 1){
				dojo.style(linktd, "display", "none");
			}else{
				dojo.style(linktd, "color", "gray");
			}
		}
	},
	
	_loadItemSet: function(){
		return new com.ibm.mm.widget.dataViewer._WB_VER_.v210.ViewConfig(this.iContext.getiWidgetAttributes(),
            this.columnManager,this.domID,this.contextRoot);
	},
	
	/*
	 * This method supports load hub data feed and invoke callback for data process, hub feed is tabular data
	 */
	renderDataFromURL: function(viewConfig){
        this.showLoadingMessage();
        var self = this;
        var feedURL = viewConfig.get('feedURL');
		if (!viewConfig.get('isWiring') || this._urlIsFromWiring) {
			/*
			 * Conditions for widget refreshing data store from url
			 * - Widget is not wired
			 * - Widget is wired, with accepting URL
			 */
	        if (!feedURL) {
	            this.showInitMessage();
	            return;
	        }
			this.currentSource = {
				feedURL: feedURL
			};
	        feedURL = this.iContext.io.rewriteURI(dojo.trim(feedURL));
	        this._dataSource = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.remoteDataSource(feedURL, function(e){
                self.urlOnChangeOnGoingMarker=false;
	            self.showErrorMessage(e);
	        });
		}
		
		if (this._dataSource) {
			this._dataSource.getData(function(data, dataSource, dataModel){
                if (self._dataReceivedLocally) {
                    self.urlOnChangeOnGoingMarker=false;
                    return;
                }//data received from locally, no need to update from url
				self.processData(data, viewConfig);
				
				var urlType = '';
				if (dataModel === 'XMLDataModel') {
					urlType = 'feed';
					if (self.publishEventEnabled['New_Feed_Address']) {
						self.iContext.iEvents.publishEvent("New Feed Address", viewConfig.get('feedURL'));
					}
				}
				else if (dataModel === 'CSVDataModel' || dataModel === 'CSVLikeDataModel') {
					urlType = 'csv';
					if (self.publishEventEnabled['New_CSV_Address']) {
						self.iContext.iEvents.publishEvent("New CSV Address", viewConfig.get('feedURL'));
					}
				}
				if (urlType !== viewConfig.get('type')) {
					var att = self.iContext.getiWidgetAttributes();
					att.setItemValue("type", urlType);
                    //save will be done when user click save button. 
                    //att.save();
				}
				//publish table
				self.publishTable(data);
				self.urlOnChangeOnGoingMarker=false;
			});
		} else {
			if (!viewConfig.get('isWiring')) { //20767: this shows the initial view of the dataviewer, don't do that if it is wired
				this.showInitMessage();
			}
			self.urlOnChangeOnGoingMarker=false;
		}
	},
	
	/*
	 * Support column dynamic event
	 */
	updateDynamicEvents: function(nameList){
		var oldDynamicEvents = this.dynamicEvents;
		var newDynamicEvents = {};
		this.dynamicEvents = {};
		
		for (var key in nameList) {
			newDynamicEvents[key] = true;
			this.dynamicEvents[key] = true;
		}
		
		for (var eventName in oldDynamicEvents) {
			if (!newDynamicEvents[eventName]) {
				this.iContext.iEvents.removeEvent(eventName);
			}
		}
		
		
	  var events = [];
      var event = null;

		for (var eventName in newDynamicEvents) {
			event = this._createEvent({
				name: eventName,
				isHandled: false,
				isPublished: true,
				localizedAttributes: {
					"en":{"title":nameList[eventName], "description":""},
					"de":{"title":nameList[eventName], "description":""},
					"zh":{"title":nameList[eventName], "description":""}
				},
				attributes: {
					"setOnNewWire": 'publishDynamicEventOnNewWire'
	            }
			});
			
		    if (event!= null) {
           		events.push(event);
       		 }

		}
		if (events.length > 0) {
         this.iContext.iEvents.setEvents(events);
      }

	},

    /**
     * Dynamic create iEvent
     * @param {Object} kwArgs
     *     name: name of iEvent
     *     type: (optional) type of iEvent, "text" by default
     *     isHandled: (optional) publish or handled, "false" by default
     *     lang: (optional) language, "en" by default
     *     handlingFn: (optional) function name of event handler
     *     localizedAttributes: (optional) 
     *     attributes: (optional) private by default
     */
    _createEvent: function(kwArgs) {
        if (!kwArgs.name || (kwArgs.isHandled && !kwArgs.handlingFn)) {
            return;
        }
        if (!kwArgs.localizedAttributes) {
            var eventShortName = kwArgs.name;
            kwArgs.localizedAttributes = {
                "en": {
                    "title": eventShortName,
                    "description": "desc_" + eventShortName
                }
            }
        }
        var obj = {
            name: null,
            type: "text",
            isHandled: false,
            lang: "en",
            handlingFn: null,
            localizedAttributes: {
                "en":{"title":"", "description":""}
            }
        };
        var args = dojo.mixin(obj, kwArgs);
        var event = this.iContext.iEvents.createEventDescription(args);

		return event;
    },
    
	/*
	 * Below methods are event handlers for data viewer's UI events in view mode and config mode
	 */
	
	cancel: function(){
		this.reload = !this.viewInitiated;
		this.iContext.iEvents.fireEvent("onModeChanged",null,{newMode:'view'});
	},
	
	/*
	 * Invoked by 'Save' button in config view, save data viewer's display setting to ItemSet
	 */
	save: function(){
        this.reload = !this.viewInitiated;
		var config = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.ViewConfig(null,this.columnManager,this.domID);
		var attr = this.iContext.getiWidgetAttributes();
        var settings = config.getChangedValue(dojo.hitch(this,this.byId),dojo.hitch(this,this.dijitById),attr,this.ColumnTable);
		
        //20330. empty value is not allowed for comparison.
        if (settings.filterColumnType === "numeric" && (isNaN(settings.filterTargetValue) || (settings.filterTargetValue === ""))) {
		    var filterPane =this.dijitById("_"+ this.iContext.widgetId + "_FILTER_PANE");
			if (filterPane.open == false) {
				filterPane._onTitleClick();
			}
			var errorMessage=this.resourceBundle.E_COMMARSION_IS_NOT_NUMBER;
			//for 20330
			this.showMessage(errorMessage,"error","messageArea");
			return ;
		}
        
        for(var name in settings){
            attr.setItemValue(name, settings[name].toString(), false);
            this.reload = true;
        }
		
//        this._urlWidgetHelper.save(); //this line of code introduced additional post request
		if(this._urlWidgetHelper.vUrl.serialize() != attr.getItemValue(this._urlWidgetHelper.attributeKey)){
            attr.setItemValue(this._urlWidgetHelper.attributeKey, this._urlWidgetHelper.vUrl.serialize());
            attr.setItemValue("feedURL", this._urlWidgetHelper.getUrl());
			this.reload = true;
        }
		if (this.iContext.getiDescriptor().getItemValue("mode") != this.iContext.constants.mode.VIEW) {
			attr.save();
			
			this._viewConfig = this._loadItemSet();
			this.iContext.iEvents.fireEvent("onModeChanged",null,{newMode:'view'});	
		}
	},
	

    _showMessage : function(message, type, nodeId) {
		if(!this.messageControl) {
			var wid = "_"+ this.iContext.widgetId + "_";
			var node = dojo.byId(wid+nodeId);
		   	this.messageControl = new com.ibm.lotus.oneui.v100.messages.Message({
				messageType: type}, node);
		}
		this.messageControl.attr(
			{
			"message" : message,
			"messageType" : type || "error"
			}
		);
		this.messageControl.show();
	},
    
	removeColumnTable: function(){
		var ColumnTableContainer = this.byId("ColumnTableContainer");		
		
		if (this.ColumnTable){
			this.ColumnTable.destroy();
			this.ColumnTable = null;

			dojo.empty(ColumnTableContainer);
			var columnTable = dojo.create("div");
			columnTable.id = this.domID+"ColumnTable"+this.getIdSuffix();
			ColumnTableContainer.appendChild(columnTable);
		}
	},
	
	/*Restore configuration view UI to default state*/
	restoreConfigView: function(){
		var selcontainer = this.byId("columnAd");
    	dojo.empty(selcontainer);
    	this.removeColumnTable();
        
		this.showMessage(false, null, "editMessageBox");
	},
	
	/*
	 * When end users click Load button on config view, data viewer will display UI for user to customize 
	 * 1) which columns to display
     * 2) the order for columns to display
     * 3) use which column for filter
	 */
	onLoadButtonClick : function(feedURL){
		if (!feedURL) {
			this.restoreConfigView();
			return;
		}
		feedURL = this.iContext.io.rewriteURI(feedURL);
		
		var self = this;
		this._dataSource = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.remoteDataSource(feedURL, function(e){
			var selcontainer = self.byId("columnAd");
        	dojo.empty(selcontainer);

			self.removeColumnTable();
			self.showErrorMessage(e, "editMessageBox");
            dojo.attr(self._urlDijit.urlInputBox.domNode,"aria-invalid","true");
		});
		this._dataSource.getData(function(data){
            dojo.attr(self._urlDijit.urlInputBox.domNode,"aria-invalid","false");
			self.restoreConfigView();
			var columnList = data.columnList;
			var selcontainer = self.byId("columnAd");
			for (var i = 0; i < columnList.length; i++) {
                var option = dojo.create('option');
                option.innerHTML = option.value = columnList[i];
                selcontainer.appendChild(option);
			}
			
			var columns = self.prepareDataForColumnListWidget(columnList);
			try{
				self.ColumnTable = new com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.ColumnsList({
					wid: self.domID + "field",
					contextRootBuilder: self.contextRoot
				}, self.byId("ColumnTable"));
				self.ColumnTable.initialize(columns);
				self.ColumnTable.render();
			}catch(e){
				self.LOGGER.severe("onLoadButtonClick", self.resourceBundle.E_CREATE_COLUMN_TABLE);
			}
		});

	},
	
	onTitleAlignClick: function(targetValue){
		var titleAlign = this.byId("titleAlign");
		var formerValue = titleAlign.value;
		if (formerValue !== targetValue) {
			var formerImg = this.byId('align_title_' + formerValue + '_img'),
			    targetImg = this.byId('align_title_' + targetValue + '_img'),
                formerBtn = this.byId('align_title_' + formerValue + "_btn"),
				targetBtn = this.byId('align_title_' + targetValue + '_btn');
            
			formerImg.src = this.contextRoot + "/images/align_" + formerValue + ".png";
			targetImg.src = this.contextRoot + "/images/align_" + targetValue + "_selected.png";

            var formerStr = dojo.string.substitute(this.resourceBundle.L_CONF_ALIGN_1, [this.resourceBundle["L_" + formerValue.toUpperCase()]]),
			    targetStr = dojo.string.substitute(this.resourceBundle.L_CONF_ALIGN_IS_SELECTED_1, [this.resourceBundle["L_" + targetValue.toUpperCase()]]);

            formerImg.alt = formerBtn.title = formerStr;
			targetImg.alt = targetBtn.title = targetStr;
			
			dojo.attr(formerImg, "aria-label", formerStr);
			dojo.attr(targetImg, "aria-label", targetStr);
			
			titleAlign.value = targetValue;
		}
	},
	
	restoreMessage: function(message) {
		var dataViewerForm = this.byId("dataViewerForm");
		var oriMessage = ""
		if (message === "invalidFeed")
		    oriMessage = this.resourceBundle.E_CHECK_URL;
		else if (message === "noContent")
		    oriMessage = this.resourceBundle.E_NO_CONTENT;
		else if (message === "allContentFiltered")
		    oriMessage = this.resourceBundle.I_ALL_CONTENT_FILTERED;
		dataViewerForm[message].value = oriMessage;
	},
	
	next: function(){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.fetchNextPage();
		}
		this.updatePageLink();
	},
	
	pre: function(){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.fetchPrePage();
		}
		this.updatePageLink();
	},
	
	getPage: function(index){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.fetchPage(index);
		}
		this.updatePageLink();
	},
	
	getServerDataPage: function(index){
		var url = this.message.first || this.message.last || this.message.previous || this.message.next;
		if (!url){ 
			//uncatched exception, indicates pipeline data is incorrect
			this.showErrorMessageTableExist(this.resourceBundle.E_ILLEGAL_PIPELINE_DATA);
			return;
		}
		
		this._viewConfig = this._loadItemSet();
		var startIndex = index * parseInt(this._viewConfig.get('rowsPerView'), 10) + 1/*opensearch spec starts with 1*/;
		var count = this._viewConfig.get('rowsPerView');
		
		var newURL = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.createPipelineURL(url, "startIndex", startIndex);
		newURL = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil.createPipelineURL(newURL, "count", count);
		
		this.handleURL({'payload' : newURL});
	},
	
	getPrePageRegion: function(){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.fetchPrePageRegion();
		}
		this.updatePageLink();
	},
	
	getNextPageRegion: function(){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (dataViewer) {
			dataViewer.fetchNextPageRegion();
		}
		this.updatePageLink();
	},
	
	updatePageLink: function(){
		var dataViewer = this.dijitById(this.domID + "dataViewerId");
		if (!dataViewer)
			return;
		var link = dataViewer.createPageLink(this.iContext.getiWidgetAttributes());
		var linktd = this.byId("pageLink");
		linktd.innerHTML = link;
	},
	
	/**
	 * Change compare filter select options
	 * @param {String} compareType "numeric" or "text"
	 */
	changeCompare: function(compareType, selectNode, defaultValue){
		var compareNode = selectNode || this.byId("contentCompare");
        dojo.empty(compareNode);
		
		var optionArray = null;
		if (compareType === 'numeric') {
			optionArray = [
				{'label' : this.resourceBundle.L_CONF_OPTION_GREATER_THAN, 'value' : "gt"},
				{'label' : this.resourceBundle.L_CONF_OPTION_GREATER_THAN_OR_EQUAL_TO, 'value' : "notlt"},
				{'label' : this.resourceBundle.L_CONF_OPTION_EQUAL_TO, 'value' : "eq"},
				{'label' : this.resourceBundle.L_CONF_OPTION_LESS_THAN_OR_EQUAL_TO, 'value' : "notgt"},
				{'label' : this.resourceBundle.L_CONF_OPTION_LESS_THAN, 'value' : "lt"}];
		} else {              // 'text' by default
			optionArray = [
				{'label' : this.resourceBundle.L_CONF_OPTION_BEGINS_WITH, 'value' : "bw"},
				{'label' : this.resourceBundle.L_CONF_OPTION_CONTAINS, 'value' : "ct"},
				{'label' : this.resourceBundle.L_CONF_OPTION_ENDS_WITH, 'value' : "ew"}];
		}
		
		if (!defaultValue) {
			defaultValue = optionArray[0].value;
		}
		for (var i=0; i<optionArray.length; i++){
            var option = dojo.create('option');
            option.innerHTML = optionArray[i].label;
			dojo.attr(option, "value", optionArray[i].value);
			if (option.value === defaultValue) {
				dojo.attr(option, "selected", "selected");
			}
            compareNode.appendChild(option);
		}
		compareNode.value = defaultValue;
	},
	
	onBoldClick: function(link) {
		var imgs = dojo.query('img', link);
		if (!imgs || !imgs.length || imgs.length <= 0) return;

		var img = imgs[0];
		
		img.src = img.src.search("bold_selected.png") === -1 ? this.contextRoot + "/images/bold_selected.png" : this.contextRoot + "/images/bold.png";
		switch(img.name){
			case "titleFontBoldImg":
				var titleFontBold = this.byId("titleFontBold");
				titleFontBold.value = (titleFontBold.value === "true") ? "false" : "true";
				img.title = img.alt = (titleFontBold.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_ENABLE_BOLD;
				break;
			case "tableFontBoldImg":
				var tableFontBold = this.byId("tableFontBold");
				tableFontBold.value = (tableFontBold.value === "true") ? "false" : "true";
				img.title = img.alt = (tableFontBold.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_FONT_ENABLE_BOLD;
				break;
			case "headerFontBoldImg":
				var headerFontBold = this.byId("headerFontBold");
				headerFontBold.value = (headerFontBold.value === "true") ? "false" : "true";
				img.title = img.alt = (headerFontBold.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_ENABLE_BOLD;
				break;
		}
	},
	
	onItalicClick: function(link) {
		var imgs = dojo.query('img', link);
		if (!imgs || !imgs.length || imgs.length <= 0) return;

		var img = imgs[0];

		img.src = img.src.search("italic_selected.png") === -1 ? this.contextRoot + "/images/italic_selected.png" : this.contextRoot + "/images/italic.png";
		switch(img.name){
			case "titleFontItalicImg":
				var titleFontItalic = this.byId("titleFontItalic");
				titleFontItalic.value = (titleFontItalic.value === "true") ? "false" : "true";
				img.title = img.alt = (titleFontItalic.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_TITLE_FONT_ENABLE_ITALIC;
				break;
			case "tableFontItalicImg":
				var tableFontItalic = this.byId("tableFontItalic");
				tableFontItalic.value = (tableFontItalic.value === "true") ? "false" : "true";
				img.title = img.alt = (tableFontItalic.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_FONT_ENABLE_ITALIC;
				break;
			case "headerFontItalicImg":
				var headerFontItalic = this.byId("headerFontItalic");
				headerFontItalic.value = (headerFontItalic.value === "true") ? "false" : "true";
				img.title = img.alt = (headerFontItalic.value === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_DISABLE_ITALIC : this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_ENABLE_ITALIC;
				break;
		}
	},
	
	onHorizontalViewClick: function(){			
		var horizontalViewRadio = this.byId("horizontalView");
		var verticalViewRadio = this.byId("verticalView");
	
		horizontalViewRadio.checked = true;
		verticalViewRadio.checked = false;
		this.byId("rowsLimit").disabled = true;
		dojo.style(this.byId("rowsLimitLabel"), "color", "#777777");
		
		this.byId("rowsPerPage").disabled = false;
		dojo.style(this.byId("rowsPerPageLabel"), "color", "#000000");
		this.setHeaderFontBold(false);
		
		this.byId("actionType_highlight").disabled = false;
		this.byId("actionType_highlight").checked = true;
	},
	
	onVerticalViewClick: function(){
	
		var horizontalViewRadio = this.byId("horizontalView");
		var verticalViewRadio = this.byId("verticalView");
		
		this.byId("rowsLimit").disabled = false;
		dojo.style(this.byId("rowsLimitLabel"), "color", "#000000");
		
		this.byId("rowsPerPage").disabled = true;
		dojo.style(this.byId("rowsPerPageLabel"), "color", "#777777");
		
		this.setHeaderFontBold(true);
		
		this.byId("actionType_highlight").disabled = true;
		this.byId("actionType_filter").checked = true;
	},
	
	setHeaderFontBold: function(flag){
		var tableHeaderFontBold = flag ? "true" : "false";
		
		this.byId("headerFontBold").value = tableHeaderFontBold;
		this.byId("headerFontBoldImg").src = (tableHeaderFontBold === "true") ? this.contextRoot + "/images/bold_selected.png" : this.contextRoot + "/images/bold.png";
		this.byId("headerFontBoldImg").title = this.byId("headerFontBoldImg").alt = (tableHeaderFontBold === "true") ? this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_DISABLE_BOLD : this.resourceBundle.L_CONF_STYLE_TABLE_HEADER_FONT_ENABLE_BOLD ;
		
	}
});
